/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainAllOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainAllOfLogicalAndSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fex", "fum", "foe", "fie", "fee")
    val toArray: Array[String] = Array("too", "you", "to", "birthday", "happy")
    
    object `when used with (contain allOf (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain allOf ("fee", "fie", "foe", "fum") and contain allOf ("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain allOf ("happy", "birthday", "to", "you") and contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain allOf ("fee", "fie", "foe", "fum") and contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain allOf ("FEE", "FIE", "FOE", "FUM") and contain allOf ("FEE", "FIE", "FUM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain allOf ("FEE", "FIE", "FOE", "FAM") and contain allOf ("FEE", "FIE", "FUM", "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FAM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain allOf ("FEE", "FIE", "FOE", "FUM") and (contain allOf ("FEE", "FIE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain allOf ("FEE", "FIE", "FOE", "FUM") and contain allOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain allOf ("FEE", "FIE", "FOE", "FAM") and contain allOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FAM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain allOf ("FEE", "FIE", "FOE", "FUM") and contain allOf ("FEE", "FIE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (contain allOf (" FEE ", " FIE ", " FOE ", " FUM ") and contain allOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) and contain allOf ("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and (contain allOf ("FEE", "FIE", "FOE", "FAM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FAM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (toArray) and contain allOf ("FEE", "FIE", "FOE", "FAM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FAM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) and contain allOf (" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) and contain allOf ("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and (contain allOf ("happy", "birthday", "to", "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (fumArray) and contain allOf ("happy", "birthday", "to", "you"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) and contain allOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) and contain allOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain allOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain allOf ("fie", "fee", "fum", "foe") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain allOf ("fee", "fie", "foe", "fum") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain allOf ("happy", "birthday", "to", "you") and legacyEqual (fumArray))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain allOf ("FIE", "FEE", "FUM", "FOE") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain allOf ("FIE", "FEE", "FAM", "FOE") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain allOf ("HAPPY", "BIRTHDAY", "TO", "YOU") and (legacyEqual (fumArray)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"HAPPY\", \"BIRTHDAY\", \"TO\", \"YOU\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain allOf ("FIE", "FEE", "FUM", "FOE") and legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain allOf ("FIE", "FEE", "FAM", "FOE") and legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain allOf ("FIE", "FEE", "FUM", "FOE") and legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain allOf (" FEE ", " FIE ", " FOE ", " FUM ") and legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain allOf xx and not contain allOf xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain allOf ("fee", "fie", "foe", "fuu") and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain allOf ("fee", "fie", "foe", "fum") and not contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain allOf ("happy", "birthday", "to", "you") and not contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\"") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain allOf ("FIE", "FEE", "FAM", "FOE") and not contain allOf ("FIE", "FEE", "FOE", "FAM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain allOf ("FIE", "FEE", "FUM", "FOE") and not contain allOf ("FIE", "FEE", "FOE", "FAM"))
        }
        checkMessageStackDepth(e1, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain allOf ("FIE", "FEE", "FAM", "FOE") and (not contain allOf ("FIE", "FEE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\"") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain allOf ("FIE", "FEE", "FAM", "FOE") and not contain allOf ("FIE", "FEE", "FOE", "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain allOf ("FIE", "FEE", "FAM", "FOE") and not contain allOf ("FIE", "FEE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\"") + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not contain allOf ("FIE", "FEE", "FUM", "FOE") and not contain allOf ("FIE", "FEE", "FOE", "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (contain allOf (" FEE ", " FIE ", " FOE ", " FUM ") and contain allOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and not contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and (not contain allOf ("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (fumArray) and not contain allOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (not contain allOf (" FEE ", " FIE ", " FOE ", " FUU ") and not contain allOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) and not contain allOf ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain allOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and not contain allOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and (not contain allOf ("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (toArray) and not contain allOf ("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedAllOfElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) and not contain allOf ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (not contain allOf (" FEE ", " FIE ", " FOE ", " FUU ") and not contain allOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `col of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(3, 2, 1, 0), Array(3, 2, 1, 0), Array(3, 2, 1, 0))
    val lists: Vector[Array[Int]] = Vector(Array(3, 2, 1, 0), Array(3, 2, 1, 0), Array(8, 4, 3, 2))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(3, 2, 1, 0), Array(3, 2, 1, 0), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("howdy", "hi", "hello"), Array("howdy", "hi", "hello"), Array("howdy", "hi", "hello"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain allOf xx and contain allOf xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain allOf (3, 2, 1) and contain allOf (1, 3, 2))
        atLeast (2, lists) should (contain allOf (3, 1, 2) and contain allOf (2, 3, 1))
        atMost (2, lists) should (contain allOf (3, 1, 2) and contain allOf (2, 3, 1))
        no (lists) should (contain allOf (3, 6, 9) and contain allOf (3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain allOf (1, 2, 3) and contain allOf (1, 3, 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " did not contain all of " + "(1, 2, 3)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain allOf (1, 2, 3) and contain allOf (1, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array(3, 2, 1, 0)) + " contained all of " + "(1, 2, 3)" + ", but " + decorateToStringValue(Array(3, 2, 1, 0)) + " did not contain all of " + "(1, 3, 4)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (contain allOf ("hi", "hello") and contain allOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"hi\", \"hello\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"ho\", \"hey\", \"howdy\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain allOf ("HELLO", "HI") and contain allOf ("HI", "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain allOf ("HO", "HELLO") and contain allOf ("HI", "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (contain allOf ("HELLO", "HI") and contain allOf ("HO", "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain allOf ("HELLO", "HI") and contain allOf ("HI", "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain allOf ("HO", "HELLO") and contain allOf ("HI", "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (contain allOf ("HELLO", "HI") and contain allOf ("HO", "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(3, 2, 1, 0)) and contain allOf (1, 3, 2))
        atLeast (2, lists) should (be (Array(3, 2, 1, 0)) and contain allOf (1, 3, 2))
        atMost (2, lists) should (be (Array(3, 2, 1, 0)) and contain allOf (2, 3, 1))
        no (lists) should (be (Array(3, 6, 9)) and contain allOf (3, 4, 5))
        no (nils) should (be (Array(1, 6, 8)) and contain allOf (1, 3, 4))
        no (listsArray) should (be (Array(2, 6, 8)) and contain allOf (3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (Array(3, 2, 1, 0)) and contain allOf (1, 3, 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " was not equal to " + decorateToStringValue(Array(3, 2, 1, 0)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (Array(3, 2, 1, 0)) and contain allOf (2, 3, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array(3, 2, 1, 0)) + " was equal to " + decorateToStringValue(Array(3, 2, 1, 0)) + ", but " + decorateToStringValue(Array(3, 2, 1, 0)) + " did not contain all of " + "(2, 3, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (Array("hey")) and contain allOf ("hello", "hi"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array("hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("howdy", "hi", "hello")) and contain allOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"ho\", \"hey\", \"howdy\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsArray) should (be (Array(3, 2, 1, 0)) and contain allOf (1, 3, 2))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array(3, 2, 1, 0)), thisLineNumber - 2, listsArray), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (Array(3, 2, 1, 0)) and contain allOf (2, 3, 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(Array(3, 2, 1, 0)) + " was equal to " + decorateToStringValue(Array(3, 2, 1, 0)) + ", but " + decorateToStringValue(Array(3, 2, 1, 0)) + " did not contain all of " + "(2, 3, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("howdy", "hi", "hello")) and contain allOf ("HELLO", "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("HI", "HELLO")) and contain allOf ("HELLO", "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was not equal to " + decorateToStringValue(Array("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("howdy", "hi", "hello")) and contain allOf ("HO", "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("howdy", "hi", "hello")) and contain allOf ("HELLO", "HI"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("HI", "HELLO")) and contain allOf ("HELLO", "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was not equal to " + decorateToStringValue(Array("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("howdy", "hi", "hello")) and contain allOf ("HO", "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain allOf xx and not contain allOf xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain allOf (3, 2, 8) and not contain allOf (8, 3, 4))
        atLeast (2, lists) should (not contain allOf (3, 8, 5) and not contain allOf (8, 3, 4))
        atMost (2, lists) should (not contain allOf (2, 4, 3) and contain allOf (4, 3, 2))
        no (list1s) should (not contain allOf (1, 2, 3) and not contain allOf (1, 3, 2))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain allOf (2, 3, 4) and not contain allOf (8, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " contained all of " + "(2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain allOf (3, 6, 8) and not contain allOf (2, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " did not contain all of " + "(3, 6, 8)" + ", but " + decorateToStringValue(Array(8, 4, 3, 2)) + " contained all of " + "(2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not contain allOf ("hello", "hi") and not contain allOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"hello\", \"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not contain allOf ("ho", "hey", "howdy") and not contain allOf ("hello", "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"ho\", \"hey\", \"howdy\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"hello\", \"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain allOf ("TO") and not contain allOf ("HO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain allOf ("HELLO", "HI") and not contain allOf ("HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not contain allOf ("HO") and not contain allOf ("HELLO", "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain allOf ("TO") and not contain allOf ("HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain allOf ("HELLO", "HI") and not contain allOf ("HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain allOf ("HO") and not contain allOf ("HELLO", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " did not contain all of " + "(\"HO\")" + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HELLO\", \"HI\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain allOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) and not contain allOf (8, 3, 4))
        atLeast (2, lists) should (not be (Array(3)) and not contain allOf (8, 3, 4))
        atMost (2, lists) should (not be (Array(4, 3, 2)) and not contain allOf (3, 4, 2))
        no (list1s) should (not be (Array(3, 2, 1)) and not contain allOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (Array(8, 4, 3, 2)) and not contain allOf (8, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " was equal to " + decorateToStringValue(Array(8, 4, 3, 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (Array(3)) and not contain allOf (2, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(8, 4, 3, 2)) + " was not equal to " + decorateToStringValue(Array(3)) + ", but " + decorateToStringValue(Array(8, 4, 3, 2)) + " contained all of " + "(2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("howdy", "hi", "hello")) and not contain allOf ("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain allOf ("hello", "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"hello\", \"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("ho")) and not contain allOf ("HO", "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("howdy", "hi", "hello")) and not contain allOf ("HELLO", "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain allOf ("HI", "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HI\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("ho")) and not contain allOf ("HO", "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("howdy", "hi", "hello")) and not contain allOf ("HELLO", "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was equal to " + decorateToStringValue(Array("howdy", "hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("ho")) and not contain allOf ("HI", "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("howdy", "hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("howdy", "hi", "hello")) + " contained all of " + "(\"HI\", \"HELLO\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
