/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainInOrderOnlyLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainInOrderOnlyLogicalOrSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fum", "fum", "foe", "fie", "fee", "fee", "fee")
    val toArray: Array[String] = Array("you", "to", "to", "birthday", "birthday", "happy")
    
    object `when used with (contain inOrderOnly xx or contain inOrderOnly xx)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain inOrderOnly ("fum", "foe", "fie", "fee") or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum") or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (contain inOrderOnly ("fum", "foe", "fie", "fee") or contain inOrderOnly ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum") or contain inOrderOnly ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or (contain inOrderOnly ("FIE", "FEE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or contain inOrderOnly ("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FIE\", \"FEE\", \"FAM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumArray should (contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE ") or contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (equal (toArray) or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (equal (fumArray) or contain inOrderOnly ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or contain inOrderOnly ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (equal (toArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (equal (fumArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (fumArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (toArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) or contain inOrderOnly (" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (legacyEqual (toArray) or contain inOrderOnly ("fum", "foe", "fie", "fee"))
        fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or contain inOrderOnly ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (legacyEqual (toArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (toArray) or contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (fumArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) or contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) or contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain inOrderOnly xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain inOrderOnly ("fum", "foe", "fie", "fee") or legacyEqual (fumArray))
        fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum") or legacyEqual (fumArray))
        fumArray should (contain inOrderOnly ("fum", "foe", "fie", "fee") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or legacyEqual (fumArray))
        fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))
        fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE ") or legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain inOrderOnly ("fee", "fie", "foe", "fum") or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not contain inOrderOnly ("fum", "foe", "fie", "fee") or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not contain inOrderOnly ("fee", "fie", "foe", "fum") or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain inOrderOnly ("fum", "foe", "fie", "fee") or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"fum\", \"foe\", \"fie\", \"fee\"") + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"fum\", \"foe\", \"fie\", \"fee\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\"") + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM") or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE") or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\"") + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not equal xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not equal (fumArray) or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not equal (toArray) or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"fum\", \"foe\", \"fie\", \"fee\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not equal (fumArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not equal (toArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (toArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (fumArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not be (fumArray) or not contain inOrderOnly ("fee", "fie", "foe", "fum"))
        fumArray should (not be (toArray) or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or not contain inOrderOnly ("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"fum\", \"foe\", \"fie\", \"fee\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not be (fumArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not be (toArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or (not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (fumArray) or not contain inOrderOnly ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (toArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) or not contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedInOrderOnlyElements", decorateToStringValue(fumArray), "\"FUM\", \"FOE\", \"FIE\", \"FEE\""), fileName, thisLineNumber - 2)
        (fumArray should (not contain inOrderOnly (Set(" FEE ", " FIE ", " FOE ", " FUU ")) or not contain inOrderOnly (Set(" FEE ", " FIE ", " FOE ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
  }
  
  object `collection of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1, 2, 2, 3), Array(1, 2, 2, 3), Array(1, 2, 2, 3))
    val lists: Vector[Array[Int]] = Vector(Array(1, 2, 2, 3), Array(1, 1, 2, 3, 3), Array(2, 3, 4))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "hello"), Array("hi", "hello"), Array("hi", "hello"))
    val toArrays: Vector[Array[String]] = Vector(Array("you", "to"), Array("you", "to"), Array("you", "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain inOrderOnly xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrderOnly (1, 2, 3) or contain inOrderOnly (1, 2, 3))
        all (list1s) should (contain inOrderOnly (3, 2, 5) or contain inOrderOnly (1, 2, 3))
        all (list1s) should (contain inOrderOnly (1, 2, 3) or contain inOrderOnly (2, 3, 4))
        
        atLeast (2, lists) should (contain inOrderOnly (1, 2, 3) or contain inOrderOnly (1, 2, 3))
        atLeast (2, lists) should (contain inOrderOnly (3, 6, 5) or contain inOrderOnly (1, 2, 3))
        atLeast (2, lists) should (contain inOrderOnly (1, 2, 3) or contain inOrderOnly (8, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrderOnly (1, 2, 3) or contain inOrderOnly (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " did not contain only " + "(1, 2, 3)" + " in order" + ", and " + decorateToStringValue(Array(2, 3, 4)) + " did not contain only " + "(1, 2, 3)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain inOrderOnly ("HI", "HELLO") or contain inOrderOnly ("hi", "hello"))
        all (hiArrays) should (contain inOrderOnly ("HELLO", "HO") or contain inOrderOnly ("hi", "hello"))
        all (hiArrays) should (contain inOrderOnly ("HI", "HELLO") or contain inOrderOnly ("hello", "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain inOrderOnly ("HELLO", "HO") or contain inOrderOnly ("hello", "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"HELLO\", \"HO\")" + " in order" + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"hello\", \"ho\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain inOrderOnly ("HI", "HELLO") or contain inOrderOnly ("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain inOrderOnly ("HELLO", "HO") or contain inOrderOnly ("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain inOrderOnly ("HI", "HELLO") or contain inOrderOnly ("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)

        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain inOrderOnly ("HELLO", "HO") or contain inOrderOnly ("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"HELLO\", \"HO\")" + " in order" + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"hello\", \"ho\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }

    object `when used with (be xx and contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(1, 2, 2, 3)) or contain inOrderOnly (1, 2, 3))
        all (list1s) should (be (Array(2, 3, 4)) or contain inOrderOnly (1, 2, 3))
        all (list1s) should (be (Array(1, 2, 2, 3)) or contain inOrderOnly (2, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Array(2, 3, 4)) or contain inOrderOnly (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1, 2, 2, 3)) + " was not equal to " + decorateToStringValue(Array(2, 3, 4)) + ", and " + decorateToStringValue(Array(1, 2, 2, 3)) + " did not contain only " + "(2, 3, 4)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("hi", "hello")) or contain inOrderOnly ("HI", "HELLO"))
        all (hiArrays) should (be (Array("ho", "hello")) or contain inOrderOnly ("HI", "HELLO"))
        all (hiArrays) should (be (Array("hi", "hello")) or contain inOrderOnly ("HELLO", "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("ho", "hello")) or contain inOrderOnly ("HELLO", "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"HELLO\", \"HI\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("hi", "hello")) or contain inOrderOnly ("HI", "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("ho", "hello")) or contain inOrderOnly ("HI", "HELLO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("hi", "hello")) or contain inOrderOnly ("HELLO", "HI"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("ho", "hello")) or contain inOrderOnly ("HELLO", "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain only " + "(\"HELLO\", \"HI\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain inOrderOnly xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain inOrderOnly (3, 2, 8) or not contain inOrderOnly (8, 3, 4))
        all (list1s) should (not contain inOrderOnly (1, 2, 3) or not contain inOrderOnly (8, 3, 4))
        all (list1s) should (not contain inOrderOnly (3, 2, 8) or not contain inOrderOnly (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain inOrderOnly (2, 3, 4) or not contain inOrderOnly (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " contained only " + "(2, 3, 4)" + " in order" + ", and " + decorateToStringValue(Array(2, 3, 4)) + " contained only " + "(2, 3, 4)" + " in order", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain inOrderOnly ("HELLO", "HI") or not contain inOrderOnly ("hello", "hi"))
        all (hiArrays) should (not contain inOrderOnly ("HI", "HELLO") or not contain inOrderOnly ("hello", "hi"))
        all (hiArrays) should (not contain inOrderOnly ("HELLO", "HI") or not contain inOrderOnly ("hi", "hello"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain inOrderOnly ("HI", "HELLO") or not contain inOrderOnly ("hi", "hello"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"HI\", \"HELLO\")" + " in order" + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"hi\", \"hello\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain inOrderOnly ("HELLO", "HI") or not contain inOrderOnly ("hello", "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain inOrderOnly ("HI", "HELLO") or not contain inOrderOnly ("hello", "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain inOrderOnly ("HELLO", "HI") or not contain inOrderOnly ("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain inOrderOnly ("HI", "HELLO") or not contain inOrderOnly ("hi", "hello"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"HI\", \"HELLO\")" + " in order" + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"hi\", \"hello\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain inOrderOnly xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) or not contain inOrderOnly (8, 3, 4))
        all (list1s) should (not be (Array(1, 2, 2, 3)) or not contain inOrderOnly (8, 3, 4))
        all (list1s) should (not be (Array(2)) or not contain inOrderOnly (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Array(1, 2, 2, 3)) or not contain inOrderOnly (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1, 2, 2, 3)) + " was equal to " + decorateToStringValue(Array(1, 2, 2, 3)) + ", and " + decorateToStringValue(Array(1, 2, 2, 3)) + " contained only " + "(1, 2, 3)" + " in order", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain inOrderOnly ("HELLO", "HO"))
        all (hiArrays) should (not be (Array("hi", "hello")) or not contain inOrderOnly ("HELLO", "HO"))
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain inOrderOnly ("HI", "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "hello")) or not contain inOrderOnly ("HI", "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"HI\", \"HELLO\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain inOrderOnly ("HELLO", "HO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hi", "hello")) or not contain inOrderOnly ("HELLO", "HO"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain inOrderOnly ("HI", "HELLO"))) (decided by upperCaseStringEquality)

        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi", "hello")) or not contain inOrderOnly ("HI", "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained only " + "(\"HI\", \"HELLO\")" + " in order", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
