/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class ArrayShouldContainOneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  val upperCaseArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a.map(_.toUpperCase) == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainOneOfLogicalOrSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fum")
    val toArray: Array[String] = Array("to")
    
    object `when used with (contain oneOf (..) or contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain oneOf ("fee", "fie", "foe", "fum") or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (contain oneOf ("fee", "fie", "foe", "fam") or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (contain oneOf ("fee", "fie", "foe", "fum") or contain oneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain oneOf ("fee", "fie", "foe", "fam") or contain oneOf ("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fam\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or contain oneOf ("fie", "fee", "fum", "foe"))
        fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or (contain oneOf ("fie", "fee", "fum", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
        (fumArray should (contain oneOf (" FEE ", " FIE ", " FOE ", " FUM ") or contain oneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (equal (toArray) or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (equal (fumArray) or contain oneOf("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (equal (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (equal (fumArray) or contain oneOf ("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or (contain oneOf ("fie", "fee", "fum", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (fumArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (toArray) or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) or contain oneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (legacyEqual (toArray) or contain oneOf("fie", "fee", "fum", "foe"))
        fumArray should (legacyEqual (fumArray) or contain oneOf("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (legacyEqual (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (legacyEqual (fumArray) or contain oneOf ("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or (contain oneOf ("fie", "fee", "fum", "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (toArray) or contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (fumArray) or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) or contain oneOf ("fie", "fee", "fum", "foe"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) or contain oneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain oneOf("fie", "fee", "fum", "foe") or legacyEqual (fumArray))
        fumArray should (contain oneOf("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))
        fumArray should (contain oneOf("fie", "fee", "fum", "foe") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))
        fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or legacyEqual (fumArray))
        fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain oneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain oneOf ("fie", "fee", "fum", "foe") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumArray), "\"fie\", \"fee\", \"fum\", \"foe\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain oneOf (" FEE ", " FIE ", " FOE ", " FUM ") or legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf ("fee", "fie", "foe", "fum"))
        fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain oneOf ("fee", "fie", "foe", "fum") or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain oneOf ("FEE", "FIE", "FOE", "FUM") or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumArray should (contain oneOf (" FEE ", " FIE ", " FOE ", " FUM ") or contain oneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not equal (fumArray) or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not equal (toArray) or not contain oneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or not contain oneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) or not contain oneOf ("fie", "fee", "fum", "foe"))
        fumArray should (not equal (fumArray) or not contain oneOf ("fie", "fee", "fum", "foe"))
        fumArray should (not equal (toArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or (not contain oneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) or not contain oneOf ("fie", "fee", "fum", "foe"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (toArray) or not contain oneOf ("fie", "fee", "fum", "foe"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (fumArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumArray should (not contain oneOf (" FEE ", " FIE ", " FOE ", " FUU ") or not contain oneOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not be (fumArray) or not contain oneOf("FEE", "FIE", "FOE", "FUM"))
        fumArray should (not be (toArray) or not contain oneOf("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or not contain oneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) or not contain oneOf ("fee", "fie", "foe", "fum"))
        fumArray should (not be (fumArray) or not contain oneOf ("fee", "fie", "foe", "fum"))
        fumArray should (not be (toArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or (not contain oneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) or not contain oneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (fumArray) or not contain oneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        (fumArray should (not be (toArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) or not contain oneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumArray), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumArray should (not contain oneOf (" FEE ", " FIE ", " FOE ", " FUU ") or not contain oneOf (" FEE ", " FIE ", " FOE ", " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(1))
    val lists: Vector[Array[Int]] = Vector(Array(1), Array(1), Array(2))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1), Array(1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi"), Array("hi"), Array("hi"))
    val toArrays: Vector[Array[String]] = Vector(Array("to"), Array("to"), Array("to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf (3, 2, 1) or contain oneOf (1, 3, 4))
        all (list1s) should (contain oneOf (3, 2, 5) or contain oneOf (1, 3, 4))
        all (list1s) should (contain oneOf (3, 2, 1) or contain oneOf (2, 3, 4))
        
        atLeast (2, lists) should (contain oneOf (3, 1, 5) or contain oneOf (1, 3, 4))
        atLeast (2, lists) should (contain oneOf (3, 6, 5) or contain oneOf (1, 3, 4))
        atLeast (2, lists) should (contain oneOf (3, 1, 5) or contain oneOf (8, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (6, 7, 8) or contain oneOf (1, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " did not contain one of (6, 7, 8), and " + decorateToStringValue(Array(2)) + " did not contain one of (1, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (contain oneOf ("ho") or contain oneOf ("he"))
        all (hiArrays) should (contain oneOf ("hi") or contain oneOf ("he"))
        all (hiArrays) should (contain oneOf ("ho") or contain oneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain oneOf ("hi") or contain oneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\"), and " + decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain oneOf ("ho") or contain oneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiArrays) should (contain oneOf ("hi") or contain oneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiArrays) should (contain oneOf ("ho") or contain oneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain oneOf ("hi") or contain oneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\"), and " + decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(1)) or contain oneOf (1, 3, 4))
        all (list1s) should (be (Array(2)) or contain oneOf (1, 3, 4))
        all (list1s) should (be (Array(1)) or contain oneOf (2, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Array(2)) or contain oneOf (2, 3, 8))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1)) + " was not equal to " + decorateToStringValue(Array(2)) + ", and " + decorateToStringValue(Array(1)) + " did not contain one of (2, 3, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (be (Array("hi")) or contain oneOf ("he"))
        all (hiArrays) should (be (Array("ho")) or contain oneOf ("he"))
        all (hiArrays) should (be (Array("hi")) or contain oneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("ho")) or contain oneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", and " + decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("hi")) or contain oneOf ("ho"))) (decided by invertedStringEquality)
        (all (hiArrays) should (be (Array("ho")) or contain oneOf ("ho"))) (decided by invertedStringEquality)
        (all (hiArrays) should (be (Array("hi")) or contain oneOf ("hi"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("ho")) or contain oneOf ("hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", and " + decorateToStringValue(Array("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain oneOf (3, 2, 8) or not contain oneOf (8, 3, 4))
        all (list1s) should (not contain oneOf (1, 2, 8) or not contain oneOf (8, 3, 4))
        all (list1s) should (not contain oneOf (3, 2, 8) or not contain oneOf (8, 3, 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf (2, 6, 8) or not contain oneOf (2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2)) + " contained one of (2, 6, 8), and " + decorateToStringValue(Array(2)) + " contained one of (2, 3, 4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (not contain oneOf ("hi") or not contain oneOf ("hi"))
        all (hiArrays) should (not contain oneOf ("ho") or not contain oneOf ("hi"))
        all (hiArrays) should (not contain oneOf ("hi") or not contain oneOf ("ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain oneOf ("ho") or not contain oneOf ("ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained one of (\"ho\"), and " + decorateToStringValue(Array("hi")) + " contained one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain oneOf ("hi") or not contain oneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiArrays) should (not contain oneOf ("ho") or not contain oneOf ("hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiArrays) should (not contain oneOf ("hi") or not contain oneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain oneOf ("ho") or not contain oneOf ("ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " contained one of (\"ho\"), and " + decorateToStringValue(Array("hi")) + " contained one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) or not contain oneOf (8, 3, 4))
        all (list1s) should (not be (Array(1)) or not contain oneOf (8, 3, 4))
        all (list1s) should (not be (Array(2)) or not contain oneOf (8, 3, 1))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Array(1)) or not contain oneOf (2, 3, 1))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(1)) + " was equal to " + decorateToStringValue(Array(1)) + ", and " + decorateToStringValue(Array(1)) + " contained one of (2, 3, 1)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiArrays) should (not be (Array("ho")) or not contain oneOf ("hi"))
        all (hiArrays) should (not be (Array("hi")) or not contain oneOf ("hi"))
        all (hiArrays) should (not be (Array("ho")) or not contain oneOf ("ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi")) or not contain oneOf ("ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")) + ", and " + decorateToStringValue(Array("hi")) + " contained one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("ho")) or not contain oneOf ("hi"))) (decided by invertedStringEquality)
        (all (hiArrays) should (not be (Array("hi")) or not contain oneOf ("hi"))) (decided by invertedStringEquality)
        (all (hiArrays) should (not be (Array("ho")) or not contain oneOf ("ho"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi")) or not contain oneOf ("ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi")) + " was equal to " + decorateToStringValue(Array("hi")) + ", and " + decorateToStringValue(Array("hi")) + " contained one of (\"ho\")", thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
