/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainTheSameElementsAsLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainTheSameElementsAsLogicalOrSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fum", "foe", "fie", "fee")
    val toArray: Array[String] = Array("you", "to", "birthday", "happy")
    
    object `when used with (contain theSameElementsAs (..) or contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain theSameElementsAs Set("fee", "fie", "foe", "fum") or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (contain theSameElementsAs Set("fee", "fie", "foe", "fam") or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (contain theSameElementsAs Set("fee", "fie", "foe", "fum") or contain theSameElementsAs Set("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsAs Set("fee", "fie", "foe", "fam") or contain theSameElementsAs Set("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fee", "fie", "foe", "fam"))) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("happy", "birthday", "to", "you"))), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FUM") or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FUM") or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or (contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FAM"))) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FUM") or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FUM") or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FAM"))) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM ") or contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (equal (toArray) or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (equal (fumArray) or contain theSameElementsAs Set("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or contain theSameElementsAs Set("happy", "birthday", "to", "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("happy", "birthday", "to", "you"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (equal (toArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (equal (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) or (contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (equal (toArray) or contain theSameElementsAs (Set("FIE", "FEE", "FAM", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) or contain theSameElementsAs (Set("FIE", "FEE", "FAM", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) or contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (legacyEqual (toArray) or contain theSameElementsAs Set("fie", "fee", "fum", "foe"))
        fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or contain theSameElementsAs Set("fie", "fee", "fam", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fie", "fee", "fam", "foe"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (legacyEqual (toArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))
        fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) or (contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (toArray) or contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE"))) (decided by upperCaseStringEquality)
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) or contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FAM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) or contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsAs (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain theSameElementsAs Set("fie", "fee", "fum", "foe") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsAs Set("fie", "fee", "fam", "foe") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsAs Set("fie", "fee", "fum", "foe") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsAs Set("fee", "fie", "foe", "fam") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fee", "fie", "foe", "fam"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE") or legacyEqual (fumArray))
        fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE") or legacyEqual (toArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FAM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FAM", "FOE") or legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        (fumArray should (contain theSameElementsAs Set("FIE", "FEE", "FUM", "FOE") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsAs Set("FEE", "FIE", "FOE", "FAM") or legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FAM"))) + ", and " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM ") or legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain theSameElementsAs (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain theSameElementsAs (Set("fee", "fie", "foe", "fuu")) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not contain theSameElementsAs (Set("fee", "fie", "foe", "fum")) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not contain theSameElementsAs (Set("fee", "fie", "foe", "fuu")) or not contain theSameElementsAs (Set("fie", "fee", "fum", "foe")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsAs (Set("fee", "fie", "foe", "fum")) or not contain theSameElementsAs (Set("fie", "fee", "fum", "foe")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fee", "fie", "foe", "fum"))) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fie", "fee", "fum", "foe"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUU")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUU")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FUM", "FOE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUU")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUU")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FUM"))) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FUM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM ") or contain theSameElementsAs Set(" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not equal (fumArray) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("fie", "fee", "fum", "foe")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or not contain theSameElementsAs (Set("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not equal (fumArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) or (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        (fumArray should (not equal (fumArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUM", "FOE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FIE", "FEE", "FUM", "FOE"))), fileName, thisLineNumber - 2)
        (fumArray should (not contain theSameElementsAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")) or not contain theSameElementsAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not be (fumArray) or not contain theSameElementsAs (Set("fie", "fee", "fuu", "foe")))
        fumArray should (not be (toArray) or not contain theSameElementsAs (Set("fee", "fie", "foe", "fum")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or not contain theSameElementsAs (Set("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not be (fumArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))
        fumArray should (not be (toArray) or not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) or (not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by upperCaseStringEquality)
        (fumArray should (not be (fumArray) or not contain theSameElementsAs (Set("FIE", "FEE", "FUU", "FOE")))) (decided by upperCaseStringEquality)
        (fumArray should (not be (toArray) or not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) or not contain theSameElementsAs (Set("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", and " + Resources("containedSameElements", decorateToStringValue(fumArray), decorateToStringValue(Set("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        (fumArray should (not contain theSameElementsAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")) or not contain theSameElementsAs (Set(" FEE ", " FIE ", " FOE ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array(3, 2, 1))
    val lists: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array(4, 3, 2))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "hello"), Array("hi", "hello"), Array("hi", "hello"))
    val toArrays: Vector[Array[String]] = Vector(Array("you", "to"), Array("you", "to"), Array("you", "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain theSameElementsAs (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsAs Set(3, 2, 1) or contain theSameElementsAs Set(1, 3, 2))
        all (list1s) should (contain theSameElementsAs Set(3, 2, 5) or contain theSameElementsAs Set(1, 3, 2))
        all (list1s) should (contain theSameElementsAs Set(3, 2, 1) or contain theSameElementsAs Set(2, 3, 4))
        
        atLeast (2, lists) should (contain theSameElementsAs Set(3, 1, 2) or contain theSameElementsAs Set(1, 2, 3))
        atLeast (2, lists) should (contain theSameElementsAs Set(3, 6, 5) or contain theSameElementsAs Set(1, 3, 2))
        atLeast (2, lists) should (contain theSameElementsAs Set(3, 1, 2) or contain theSameElementsAs Set(8, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsAs Set(3, 1, 2) or contain theSameElementsAs Set(1, 3, 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(4, 3, 2)) + " did not contain the same elements as " + decorateToStringValue(Set(3, 1, 2)) + ", and " + decorateToStringValue(Array(4, 3, 2)) + " did not contain the same elements as " + decorateToStringValue(Set(1, 3, 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HI") or contain theSameElementsAs Set("hello", "hi"))
        all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HO") or contain theSameElementsAs Set("hello", "hi"))
        all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HI") or contain theSameElementsAs Set("hello", "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HO") or contain theSameElementsAs Set("hello", "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("HELLO", "HO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("hello", "ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HI") or contain theSameElementsAs Set("hello", "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HO") or contain theSameElementsAs Set("hello", "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HI") or contain theSameElementsAs Set("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain theSameElementsAs Set("HELLO", "HO") or contain theSameElementsAs Set("hello", "ho"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("HELLO", "HO")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("hello", "ho")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain theSameElementsAs (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(3, 2, 1)) or contain theSameElementsAs Set(1, 2, 3))
        all (list1s) should (be (Array(2, 3, 4)) or contain theSameElementsAs Set(1, 2, 3))
        all (list1s) should (be (Array(3, 2, 1)) or contain theSameElementsAs Set(2, 3, 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Array(2, 3, 4)) or contain theSameElementsAs Set(2, 3, 4))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(3, 2, 1)) + " was not equal to " + decorateToStringValue(Array(2, 3, 4)) + ", and " + decorateToStringValue(Array(3, 2, 1)) + " did not contain the same elements as " + decorateToStringValue(Set(2, 3, 4)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsAs Set("HELLO", "HI"))
        all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsAs Set("HELLO", "HI"))
        all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsAs Set("HELLO", "HO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsAs Set("HELLO", "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("HELLO", "HO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsAs Set("HELLO", "HI"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsAs Set("HELLO", "HI"))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (be (Array("hi", "hello")) or contain theSameElementsAs Set("HELLO", "HO"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("ho", "hello")) or contain theSameElementsAs Set("HELLO", "HO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements as " + decorateToStringValue(Set("HELLO", "HO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain theSameElementsAs xx and not contain theSameElementsAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain theSameElementsAs (Set(3, 2, 8)) or not contain theSameElementsAs (Set(8, 3, 4)))
        all (list1s) should (not contain theSameElementsAs (Set(1, 2, 3)) or not contain theSameElementsAs (Set(8, 3, 4)))
        all (list1s) should (not contain theSameElementsAs (Set(3, 2, 8)) or not contain theSameElementsAs (Set(2, 3, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsAs (Set(4, 2, 3)) or not contain theSameElementsAs (Set(2, 3, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(4, 3, 2)) + " contained the same elements as " + decorateToStringValue(Set(4, 2, 3)) + ", and " + decorateToStringValue(Array(4, 3, 2)) + " contained the same elements as " + decorateToStringValue(Set(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HO")) or not contain theSameElementsAs (Set("hello", "ho")))
        all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HI")) or not contain theSameElementsAs (Set("hello", "ho")))
        all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HO")) or not contain theSameElementsAs (Set("hello", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HI")) or not contain theSameElementsAs (Set("hello", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("HELLO", "HI")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("hello", "hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HO")) or not contain theSameElementsAs (Set("hello", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HI")) or not contain theSameElementsAs (Set("hello", "ho")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HO")) or not contain theSameElementsAs (Set("hello", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain theSameElementsAs (Set("HELLO", "HI")) or not contain theSameElementsAs (Set("hello", "hi")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("HELLO", "HI")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("hello", "hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (...) and not contain theSameElementsAs (...))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) or not contain theSameElementsAs (Set(8, 3, 4)))
        all (list1s) should (not be (Array(3, 2, 1)) or not contain theSameElementsAs (Set(8, 3, 4)))
        all (list1s) should (not be (Array(2)) or not contain theSameElementsAs (Set(1, 2, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Array(3, 2, 1)) or not contain theSameElementsAs (Set(2, 3, 1)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array(3, 2, 1)) + " was equal to " + decorateToStringValue(Array(3, 2, 1)) + ", and " + decorateToStringValue(Array(3, 2, 1)) + " contained the same elements as " + decorateToStringValue(Set(2, 3, 1)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsAs (Set("HELLO", "HO")))
        all (hiArrays) should (not be (Array("hello", "hi")) or not contain theSameElementsAs (Set("HELLO", "HO")))
        all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsAs (Set("HELLO", "HI")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsAs (Set("HELLO", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsAs (Set("HELLO", "HO")))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hello", "hi")) or not contain theSameElementsAs (Set("HELLO", "HO")))) (decided by upperCaseStringEquality)
        (all (hiArrays) should (not be (Array("hello", "ho")) or not contain theSameElementsAs (Set("HELLO", "HI")))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi", "hello")) or not contain theSameElementsAs (Set("HELLO", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", and " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements as " + decorateToStringValue(Set("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
