/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class ArrayShouldContainTheSameElementsInOrderAsLogicalAndSpec extends Spec with Matchers {
  
  val invertedArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseArrayOfStringEquality = 
    new Equality[Array[String]] {
      def areEqual(a: Array[String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  //ADDITIONAL//
  
  val fileName: String = "ArrayShouldContainTheSameElementsInOrderAsLogicalAndSpec.scala"
  
  object `a Array` {
    
    val fumArray: Array[String] = Array("fum", "foe", "fie", "fee")
    val toArray: Array[String] = Array("you", "to", "birthday", "happy")
    
    object `when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") and contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsInOrderAs Set(" FUM ", " FOE ", " FIE ", " FEE ") and contain theSameElementsInOrderAs Set(" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (equal (fumArray) and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (equal (fumArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (equal (toArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (equal (fumArray) and (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FAM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FAM"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (equal (toArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (equal (toArray) and contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (equal (fumArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (equal (toArray) and contain theSameElementsInOrderAs Set(" FUM ", " FOE ", " FIE ", " FEE "))) (decided by invertedArrayOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        val e1 = intercept[TestFailedException] {
          fumArray should (legacyEqual (toArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (legacyEqual (fumArray) and (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (legacyEqual (toArray) and contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (legacyEqual (fumArray) and contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fum", "foe", "fie", "fee") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("fee", "fie", "foe", "fum") and legacyEqual (fumArray))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and legacyEqual (fumArray))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") and legacyEqual (toArray))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (contain theSameElementsInOrderAs LinkedList("HAPPY", "BIRTHDAY", "TO", "YOU") and (legacyEqual (fumArray)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("HAPPY", "BIRTHDAY", "TO", "YOU"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FEE", "FIE", "FOE", "FUM") and legacyEqual (fumArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (contain theSameElementsInOrderAs LinkedList("FUM", "FOE", "FIE", "FEE") and legacyEqual (toArray))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))) + ", but " + Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
        (fumArray should (contain theSameElementsInOrderAs LinkedList(" FUM ", " FOE ", " FIE ", " FEE ") and legacyEqual (fumArray))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")) and not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")) and not contain theSameElementsInOrderAs (LinkedList("happy", "birthday", "to", "you")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")) and not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fee", "fie", "foe", "fum"))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) and (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")) and not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FEE", "FIE", "FOE", "FUM"))) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not equal xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not equal (toArray) and not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not equal (toArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not equal (fumArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not equal (toArray) and (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not equal (fumArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not equal (fumArray) and not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumArray), decorateToStringValue(fumArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not equal (toArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by invertedArrayOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumArray), decorateToStringValue(toArray)), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (not be (toArray) and not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain theSameElementsInOrderAs (LinkedList("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and not contain theSameElementsInOrderAs (LinkedList("fum", "foe", "fie", "fee")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("fum", "foe", "fie", "fee"))), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (not be (toArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumArray should (not be (fumArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumArray should (not be (toArray) and (not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumArray should (not be (toArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumArray should (not be (toArray) and not contain theSameElementsInOrderAs (LinkedList("FUM", "FOE", "FIE", "FEE")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumArray), decorateToStringValue(toArray)) + ", but " + Resources("containedSameElementsInOrder", decorateToStringValue(fumArray), decorateToStringValue(LinkedList("FUM", "FOE", "FIE", "FEE"))), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumArray should (not be (fumArray) and not contain theSameElementsInOrderAs (LinkedList("FEE", "FIE", "FOE", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumArray), decorateToStringValue(fumArray)), fileName, thisLineNumber - 2)
        (fumArray should (not contain theSameElementsInOrderAs (LinkedList(" FEE ", " FIE ", " FOE ", " FUU ")) and not contain theSameElementsInOrderAs (LinkedList(" FEE ", " FIE ", " FOE ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `col of Arrays` {
    
    val list1s: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array(1, 2, 3))
    val lists: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array(2, 3, 4))
    val nils: Vector[Array[Int]] = Vector(Array(), Array(), Array())
    val listsArray: Vector[Array[Int]] = Vector(Array(1, 2, 3), Array(1, 2, 3), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "hello"), Array("hi", "hello"), Array("hi", "hello"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atMost (2, lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        no (lists) should (contain theSameElementsInOrderAs LinkedList(3, 6, 9) and contain theSameElementsInOrderAs LinkedList(3, 4, 5))
        no (nils) should (contain theSameElementsInOrderAs LinkedList(1, 2, 8) and contain theSameElementsInOrderAs LinkedList(1, 3, 4))
        no (listsArray) should (contain theSameElementsInOrderAs LinkedList(3, 8, 5) and contain theSameElementsInOrderAs LinkedList(3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 2, 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain theSameElementsInOrderAs LinkedList(1, 2, 3) and contain theSameElementsInOrderAs LinkedList(1, 3, 4))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array(1, 2, 3)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 2, 3)) + ", but " + decorateToStringValue(Array(1, 2, 3)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(1, 3, 4)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain theSameElementsInOrderAs LinkedList("hi", "hello") and contain theSameElementsInOrderAs LinkedList("ho", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array()) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("hi", "hello") and contain theSameElementsInOrderAs LinkedList("hello", "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hello", "hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HO", "HELLO") and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HO", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") and contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HO", "HELLO") and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HO", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (contain theSameElementsInOrderAs LinkedList("HI", "HELLO") and contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be xx and contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atLeast (2, lists) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        atMost (2, lists) should (be (Array(3, 2, 1)) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        no (lists) should (be (Array(3, 6, 9)) and contain theSameElementsInOrderAs LinkedList(3, 4, 5))
        no (nils) should (be (Array(1, 6, 8)) and contain theSameElementsInOrderAs LinkedList(1, 3, 4))
        no (listsArray) should (be (Array(2, 6, 8)) and contain theSameElementsInOrderAs LinkedList(3, 4, 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " was not equal to " + decorateToStringValue(Array(1, 2, 3)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(2, 3, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array(1, 2, 3)) + " was equal to " + decorateToStringValue(Array(1, 2, 3)) + ", but " + decorateToStringValue(Array(1, 2, 3)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 8)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (Array("hey")) and contain theSameElementsInOrderAs LinkedList("hi", "hello"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array("hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("hi", "hello")) and contain theSameElementsInOrderAs LinkedList("hello", "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hello", "hi")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsArray) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(1, 2, 3))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(Array()) + " was not equal to " + decorateToStringValue(Array(1, 2, 3)), thisLineNumber - 2, listsArray), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (Array(1, 2, 3)) and contain theSameElementsInOrderAs LinkedList(2, 3, 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(Array(1, 2, 3)) + " was equal to " + decorateToStringValue(Array(1, 2, 3)) + ", but " + decorateToStringValue(Array(1, 2, 3)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 8)), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (be (Array("hi", "hello")) and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("HI", "HELLO")) and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (be (Array("hi", "hello")) and contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (be (Array("hi", "hello")) and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("HI", "HELLO")) and contain theSameElementsInOrderAs LinkedList("HI", "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (be (Array("hi", "hello")) and contain theSameElementsInOrderAs LinkedList("HELLO", "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList(3, 2, 8)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        atLeast (2, lists) should (not contain theSameElementsInOrderAs (LinkedList(3, 8, 5)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        atMost (2, lists) should (not contain theSameElementsInOrderAs (LinkedList(2, 4, 3)) and contain theSameElementsInOrderAs (LinkedList(2, 3, 4)))
        no (list1s) should (not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)) and not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList(3, 6, 8)) and not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(3, 6, 8)) + ", but " + decorateToStringValue(Array(2, 3, 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("hi", "hello")) and not contain theSameElementsInOrderAs (LinkedList("ho", "hey", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("ho", "hey", "howdy")) and not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("ho", "hey", "howdy")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) and not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) and not contain theSameElementsInOrderAs (LinkedList("HO")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) and not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) and not contain theSameElementsInOrderAs (LinkedList("HELLO", "HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")) and not contain theSameElementsInOrderAs (LinkedList("HO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")) and not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HELLO", "HI")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be xx and not contain theSameElementsInOrderAs xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Array(2)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        atLeast (2, lists) should (not be (Array(3)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        atMost (2, lists) should (not be (Array(2, 3, 4)) and not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)))
        no (list1s) should (not be (Array(1, 2, 3)) and not contain theSameElementsInOrderAs (LinkedList(1, 2, 3)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (Array(2, 3, 4)) and not contain theSameElementsInOrderAs (LinkedList(8, 3, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " was equal to " + decorateToStringValue(Array(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (Array(3)) and not contain theSameElementsInOrderAs (LinkedList(2, 3, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Array(2, 3, 4)) + " was not equal to " + decorateToStringValue(Array(3)) + ", but " + decorateToStringValue(Array(2, 3, 4)) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList(2, 3, 4)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "hello")) and not contain theSameElementsInOrderAs (LinkedList("ho", "hey", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain theSameElementsInOrderAs (LinkedList("hi", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiArrays) should (not be (Array("ho")) and not contain theSameElementsInOrderAs (LinkedList("HO", "HELLO")))
        
        val e1 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("hi", "hello")) and not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiArrays) should (not be (Array("ho")) and not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (not be (Array("ho")) and not contain theSameElementsInOrderAs (LinkedList("HO", "HELLO")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("hi", "hello")) and not contain theSameElementsInOrderAs (LinkedList("HELLO", "HI")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was equal to " + decorateToStringValue(Array("hi", "hello")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiArrays) should (not be (Array("ho")) and not contain theSameElementsInOrderAs (LinkedList("HI", "HELLO")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Array("hi", "hello")) + " was not equal to " + decorateToStringValue(Array("ho")) + ", but " + decorateToStringValue(Array("hi", "hello")) + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList("HI", "HELLO")), thisLineNumber - 2, hiArrays), fileName, thisLineNumber - 2)
      }
    }
  }
}
