/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAllOfSpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
def javaMap[K, V](elements: Entry[K, V]*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e.getKey, e.getValue))
    m
  }
import org.scalatest.Entry

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fex", "fex"), Entry("fum", "fum"), Entry("foe", "foe"), Entry("fie", "fie"), Entry("fee", "fee"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("too", "too"), Entry("you", "you"), Entry("to", "to"), Entry("birthday", "birthday"), Entry("happy", "happy"))

    object `when used with contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain allOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should contain allOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))
        intercept[TestFailedException] {
          fumjavaMap should contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain allOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should contain allOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))
        }
        (fumjavaMap should contain allOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain allOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain allOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        intercept[TestFailedException] {
          fumjavaMap should (contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain allOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should (contain allOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))
        }
        (fumjavaMap should (contain allOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should not contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should not contain allOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElements", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaMap should not contain allOf ("happy", "birthday", "to", "you", "dear")
        intercept[TestFailedException] {
          tojavaMap should not contain allOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should not contain allOf ("happy", "birthday", "to", "you", "dear")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should not contain allOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU"))) (decided by upperCaseStringEquality)
        }
        tojavaMap should not contain allOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))
        intercept[TestFailedException] {
          (tojavaMap should not contain allOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should (not contain allOf ("HAPPY", "BIRTHDAY", "TO", "YOU", "DEAR"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should (not contain allOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElements", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        tojavaMap should (not contain allOf ("NICE", "TO", "MEET", "YOU", "TOO"))
        intercept[TestFailedException] {
          tojavaMap should (not contain allOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should (not contain allOf ("NICE", "TO", "MEET", "YOU", "TOO"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should (not contain allOf (Entry("HAPPY", "HAPPY"), Entry("BIRTHDAY", "BIRTHDAY"), Entry("TO", "TO"), Entry("YOU", "YOU")))) (decided by upperCaseStringEquality)
        }
        tojavaMap should (not contain allOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))
        intercept[TestFailedException] {
          (tojavaMap should (not contain allOf (Entry(" HAPPY ", " HAPPY "), Entry(" BIRTHDAY ", " BIRTHDAY "), Entry(" TO ", " TO "), Entry(" YOU ", " YOU ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap(Entry(8, 8), Entry(4, 4), Entry(3, 3), Entry(2, 2)))
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap(Entry(3, 3), Entry(2, 2), Entry(1, 1), Entry(0, 0)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("howdy", "howdy"), Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("howdy", "howdy"), Entry("hi", "hi"), Entry("he", "he")), javaMap(Entry("howdy", "howdy"), Entry("hi", "hi"), Entry("he", "he")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("happy", "happy"), Entry("to", "to"), Entry("you", "you")), javaMap(Entry("happy", "happy"), Entry("to", "to"), Entry("you", "you")), javaMap(Entry("happy", "happy"), Entry("to", "to"), Entry("you", "you")))

    object `when used with contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atLeast (2, lists) should contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        atMost (2, lists) should contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        no (lists) should contain allOf (Entry(3, 3), Entry(4, 4), Entry(5, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(8, 8), Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain all of " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain allOf (Entry("he", "he"), Entry("hi", "hi"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain allOf (Entry("ho", "ho"), Entry("hi", "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaMaps) should contain allOf (Entry("HE", "HE"), Entry("HI", "HI"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain allOf (Entry("HO", "HO"), Entry("HI", "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain allOf (Entry("HE", "HE"), Entry("HI", "HI"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain allOf (Entry("HO", "HO"), Entry("HI", "HI"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should contain allOf (Entry("he", "he"), Entry("hi", "hi"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain allOf (Entry("ho", "ho"), Entry("hi", "hi"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

    object `when used with (contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atLeast (2, lists) should (contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        atMost (2, lists) should (contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        no (lists) should (contain allOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain allOf (Entry(1, 1), Entry(2, 2), Entry(3, 3)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(8, 8), Entry(4, 4), Entry(3, 3), Entry(2, 2))) + " did not contain all of " + "(1=1, 2=2, 3=3)" + " (JavaMapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain allOf (Entry("he", "he"), Entry("hi", "hi")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain allOf (Entry("ho", "ho"), Entry("hi", "hi")))
        }
        implicit val ise = upperCaseStringEquality
        all (hijavaMaps) should (contain allOf (Entry("HE", "HE"), Entry("HI", "HI")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain allOf (Entry("HO", "HO"), Entry("HI", "HI")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain allOf (Entry("HE", "HE"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain allOf (Entry("HO", "HO"), Entry("HI", "HI")))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hijavaMaps) should (contain allOf (Entry("he", "he"), Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain allOf (Entry("ho", "ho"), Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

    object `when used with not contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain allOf (Entry("you", "you"), Entry("to", "to"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("happy", "happy"), Entry("to", "to"), Entry("you", "you"))) + " contained all of " + "(you=you, to=to)" +  " (JavaMapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should not contain allOf ("NICE", "MEET", "YOU")
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain allOf (Entry("YOU", "YOU"), Entry("TO", "TO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain allOf ("NICE", "MEET", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain allOf (Entry("YOU", "YOU"), Entry("TO", "TO"))) (decided by upperCaseStringEquality)
        }
        all (tojavaMaps) should not contain allOf (Entry(" YOU ", " YOU "), Entry(" TO ", " TO "))
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain allOf (Entry(" YOU ", " YOU "), Entry(" TO ", " TO "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain allOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain allOf (Entry("you", "you"), Entry("to", "to")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("happy", "happy"), Entry("to", "to"), Entry("you", "you"))) + " contained all of " + "(you=you, to=to)" + " (JavaMapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (tojavaMaps) should (not contain allOf ("NICE", "MEET", "YOU"))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain allOf (Entry("YOU", "YOU"), Entry("TO", "TO")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain allOf ("NICE", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain allOf (Entry("YOU", "YOU"), Entry("TO", "TO")))) (decided by upperCaseStringEquality)
        }
        all (tojavaMaps) should (not contain allOf (Entry(" YOU ", " YOU "), Entry(" TO ", " TO ")))
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain allOf (Entry(" YOU ", " YOU "), Entry(" TO ", " TO ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
