/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtLeastOneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
def javaMap[K, V](elements: Entry[K, V]*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e.getKey, e.getValue))
    m
  }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainAtLeastOneOfLogicalAndSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"))
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain atLeastOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) and contain atLeastOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (equal (fumjavaMap) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (tojavaMap) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) and contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) and contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain atLeastOneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and legacyEqual (fumjavaMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")) and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and (legacyEqual (fumjavaMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atLeastOneOf (Entry("have", "have"), Entry("a", "a"), Entry("nice", "nice"), Entry("day", "day")) and legacyEqual (fumjavaMap))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and legacyEqual (fumjavaMap))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and legacyEqual (tojavaMap))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "fum=fum, fum=fum, fum=fum") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) and contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not equal (tojavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (fumjavaMap) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) and not contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not be (tojavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (tojavaMap) and not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) and not contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) and not contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to")), javaMap(Entry("to", "to")), javaMap(Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atLeastOneOf (Entry(3, 3), Entry(1, 1), Entry(5, 5)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (contain atLeastOneOf (Entry(3, 3), Entry(2, 2), Entry(8, 8)) and contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (contain atLeastOneOf (Entry(3, 3), Entry(6, 6), Entry(9, 9)) and contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain atLeastOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain atLeastOneOf(Entry(3, 3), Entry(8, 8), Entry(5, 5)) and contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (Entry(1, 1), Entry(6, 6), Entry(8, 8)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (1=1, 6=6, 8=8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " contained at least one of (1=1, 2=2, 8=8), but " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (1=1, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf (Entry("hi", "hi"), Entry("hello", "hello")) and contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap()) + " did not contain at least one of (hi=hi, hello=hello)", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi"), Entry("hello", "hello")) and contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (hi=hi, hello=hello), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (ho=ho, hey=hey, howdy=howdy)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Map()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(javaMap()) + " did not contain at least one of (1=1, 3=3, 4=4)", thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " contained at least one of (1=1, 2=2, 8=8), but " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (1=1, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")) and contain atLeastOneOf (Entry("he", "he")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")) and contain atLeastOneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")) and contain atLeastOneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")) and contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")) and contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")) and contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (be (javaMap(Entry(8, 8))) and contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (be (javaMap(Entry(8, 8))) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (be (javaMap(Entry(8, 8))) and contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " was not equal to " + decorateToStringValue(javaMap(Entry(1, 1))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1))) + ", but " + decorateToStringValue(javaMap(Entry(1, 1))) + " did not contain at least one of (2=2, 3=3, 8=8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaMap(Entry("hey", "hey"))) and contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap(Entry("hey", "hey"))), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) and contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (ho=ho, hey=hey, howdy=howdy)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Map()))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap(Entry(1, 1))), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(1, 1))) and contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1))) + ", but " + decorateToStringValue(javaMap(Entry(1, 1))) + " did not contain at least one of (2=2, 3=3, 8=8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) and contain atLeastOneOf (Entry("he", "he")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) and contain atLeastOneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) and contain atLeastOneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) and contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) and contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) and contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atLeastOneOf (Entry(3, 3), Entry(2, 2), Entry(8, 8)) and not contain atLeastOneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not contain atLeastOneOf (Entry(3, 3), Entry(8, 8), Entry(5, 5)) and not contain atLeastOneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not contain atLeastOneOf (Entry(3, 3), Entry(6, 6), Entry(8, 8)) and contain atLeastOneOf (Entry(5, 5), Entry(3, 3), Entry(4, 4)))
        no (lists) should (not contain atLeastOneOf (Entry(1, 1), Entry(2, 2), Entry(9, 9)) and not contain atLeastOneOf (Entry(2, 2), Entry(1, 1), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) and not contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " contained at least one of (2=2, 6=6, 8=8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (Entry(3, 3), Entry(6, 6), Entry(8, 8)) and not contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (3=3, 6=6, 8=8), but " + decorateToStringValue(javaMap(Entry(2, 2))) + " contained at least one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atLeastOneOf (Entry("hi", "hi"), Entry("hello", "hello")) and not contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")) and not contain atLeastOneOf (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (ho=ho, hey=hey, howdy=howdy), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not contain atLeastOneOf (Entry("hi", "hi")) and not contain atLeastOneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atLeastOneOf (Entry("ho", "ho")) and not contain atLeastOneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atLeastOneOf (Entry("hi", "hi")) and not contain atLeastOneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain atLeastOneOf (Entry("hi", "hi")) and not contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atLeastOneOf (Entry("ho", "ho")) and not contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atLeastOneOf (Entry("hi", "hi")) and not contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain at least one of (hi=hi), but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2))) and not contain atLeastOneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (not be (javaMap(Entry(3, 3))) and not contain atLeastOneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (not be (javaMap(Entry(3, 3))) and contain atLeastOneOf (Entry(5, 5), Entry(3, 3), Entry(4, 4)))
        no (list1s) should (not be (javaMap(Entry(1, 1))) and not contain atLeastOneOf (Entry(2, 2), Entry(1, 1), Entry(5, 5)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(2, 2))) and not contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " was equal to " + decorateToStringValue(javaMap(Entry(2, 2))), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(Entry(3, 3))) and not contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " was not equal to " + decorateToStringValue(javaMap(Entry(3, 3))) + ", but " + decorateToStringValue(javaMap(Entry(2, 2))) + " contained at least one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) and not contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain atLeastOneOf (Entry("hi", "hi"), Entry("hello", "hello")))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (hi=hi, hello=hello)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain atLeastOneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) and not contain atLeastOneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain atLeastOneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) and not contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) and not contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", but " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained at least one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
