/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtLeastOneOfSpec extends Spec with Matchers {

  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
def javaMap[K, V](elements: Entry[K, V]*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e.getKey, e.getValue))
    m
  }
import org.scalatest.Entry

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"))

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        intercept[TestFailedException] {
          fumjavaMap should contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"))) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))
        }
        (fumjavaMap should contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf (Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))
        }
        (fumjavaMap should (contain atLeastOneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
    }

/*
 I purposely don't want to support this syntax:

        fumjavaMap should contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        tojavaMap should not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to"))
        intercept[TestFailedException] {
          tojavaMap should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) (decided by invertedStringEquality)
        }
        tojavaMap should not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "))
        intercept[TestFailedException] {
          (tojavaMap should not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "))) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the top one does happen to compile and run:

        tojavaMap should not contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        // tojavaMap should not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
        // tojavaMap should (not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))))

The bottom two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          tojavaMap should (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(tojavaMap), "happy=happy, birthday=birthday, to=to, you=you"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        tojavaMap should (not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to")))
        intercept[TestFailedException] {
          tojavaMap should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should (not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to")))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (tojavaMap should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        }
        tojavaMap should (not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO ")))
        intercept[TestFailedException] {
          (tojavaMap should (not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a collection of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to")), javaMap(Entry("to", "to")), javaMap(Entry("to", "to")))

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        atLeast (2, lists) should contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        atMost (2, lists) should contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4))
        no (lists) should contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5))
        no (nils) should contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        no (listsMap) should contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (1=1, 3=3, 4=4) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy"))
        }
        e2.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap()) + " did not contain at least one of (ho=ho, hey=hey, howdy=howdy) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap()) + " did not contain at least one of (1=1, 3=3, 4=4) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain atLeastOneOf (Entry("hi", "hi"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain atLeastOneOf (Entry("ho", "ho"))
        }
        implicit val ise = invertedStringEquality
        all (hijavaMaps) should contain atLeastOneOf (Entry("ho", "ho"))
        intercept[TestFailedException] {
          all (hijavaMaps) should contain atLeastOneOf (Entry("hi", "hi"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain atLeastOneOf (Entry("ho", "ho"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain atLeastOneOf (Entry("hi", "hi"))) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hijavaMaps) should contain atLeastOneOf (Entry("hi", "hi"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain atLeastOneOf (Entry("ho", "ho"))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atMost (2, lists) should (contain atLeastOneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        no (lists) should (contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))
        no (nils) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        no (listsMap) should (contain atLeastOneOf (Entry(3, 3), Entry(4, 4), Entry(5, 5)))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain at least one of (1=1, 3=3, 4=4) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf (Entry("ho", "ho"), Entry("hey", "hey"), Entry("howdy", "howdy")))
        }
        e2.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap()) + " did not contain at least one of (ho=ho, hey=hey, howdy=howdy) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should (contain atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap()) + " did not contain at least one of (1=1, 3=3, 4=4) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")))
        }
        implicit val ise = invertedStringEquality
        all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hijavaMaps) should (contain atLeastOneOf (Entry("hi", "hi")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atLeastOneOf (Entry("ho", "ho")))) (decided by defaultEquality[java.util.Map.Entry[String, String]])
        }
      }
    }

/*
 I purposely don't want to support this syntax:

scala> all (list1s) should contain (atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should contain (atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
                                             ^

scala> all (list1s) should (contain (atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should (contain (atLeastOneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4))))
                                              ^

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("to", "to"))) + " contained at least one of (happy=happy, birthday=birthday, to=to, you=you) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (tojavaMaps) should not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to"))
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to"))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))) (decided by invertedStringEquality)
        }
        all (tojavaMaps) should not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "))
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "))) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the last one does happen to compile and run:

scala> all (tojavaMaps) should (not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (tojavaMaps) should (not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))))
                                                    ^

scala> all (tojavaMaps) should not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (tojavaMaps) should not (contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum"))))
                                                   ^

scala> all (tojavaMaps) should not contain (atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))

The top two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain atLeastOneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap(Entry("to", "to"))) + " contained at least one of (happy=happy, birthday=birthday, to=to, you=you) (JavaMapShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (tojavaMaps) should (not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to")))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain atLeastOneOf (Entry("to", "to"), Entry("to", "to"), Entry("to", "to"), Entry("to", "to")))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain atLeastOneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by invertedStringEquality)
        }
        all (tojavaMaps) should (not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO ")))
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain atLeastOneOf (Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO "), Entry(" TO ", " TO ")))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
