/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class JavaMapShouldContainOneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = a != b
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[java.util.Map.Entry[String, String]] {
      def areEqual(a: java.util.Map.Entry[String, String], b: Any): Boolean = b match {
    case entry: java.util.Map.Entry[_, _] => 
      (entry.getKey, entry.getValue) match {
        case (k: String, v: String) => a.getKey.toUpperCase == k && a.getValue.toUpperCase == v
        case _ => (a.getKey.toUpperCase, a.getValue.toUpperCase) == b
      }
    case _ => (a.getKey.toUpperCase, a.getValue.toUpperCase) == b
  }
    }
  
  val upperCasejavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a.asScala.map(e => upperCase(e)) == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.toLowerCase, s.getValue.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
            case (k: String, v: String) => normalized(org.scalatest.Entry(k, v))
            case _ => b
        }
        case _ => b
      }
    }
val trimmed: Normalization[java.util.Map.Entry[String, String]] = new Normalization[java.util.Map.Entry[String, String]] {
    def normalized(s: java.util.Map.Entry[String, String]): java.util.Map.Entry[String, String] = org.scalatest.Entry(s.getKey.trim, s.getValue.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case entry: java.util.Map.Entry[_, _] => 
          (entry.getKey, entry.getValue) match {
             case (k: String, v: String) => normalized(Entry(k, v))
            case _ => b
          }
        case _ => b
      }
    }
def javaMap[K, V](elements: Entry[K, V]*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e.getKey, e.getValue))
    m
  }
import org.scalatest.Entry
  
  val fileName: String = "JavaMapShouldContainOneOfLogicalOrSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap(Entry("fum", "fum"))
    val tojavaMap: java.util.Map[String, String] = javaMap(Entry("to", "to"))
    
    object `when used with (contain oneOf (..) or contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fam", "fam"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fam", "fam")) or contain oneOf (Entry("happy", "happy"), Entry("birthday", "birthday"), Entry("to", "to"), Entry("you", "you")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fam=fam") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "happy=happy, birthday=birthday, to=to, you=you"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe"))))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (tojavaMap) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (equal (fumjavaMap) or contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (equal (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (equal (fumjavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (fumjavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (equal (tojavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) or contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (legacyEqual (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe"))))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (tojavaMap) or contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) or contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe"), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))
        fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        (fumjavaMap should (contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")) or legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "fie=fie, fee=fee, fum=fum, foe=foe") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")) or contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUM ", " FUM ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not equal (fumjavaMap) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not equal (tojavaMap) or not contain oneOf(Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) or not contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not equal (fumjavaMap) or not contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))
        fumjavaMap should (not equal (tojavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) or not contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (tojavaMap) or not contain oneOf (Entry("fie", "fie"), Entry("fee", "fee"), Entry("fum", "fum"), Entry("foe", "foe")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        (fumjavaMap should (not equal (fumjavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) or not contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not be (fumjavaMap) or not contain oneOf(Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        fumjavaMap should (not be (tojavaMap) or not contain oneOf(Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "fee=fee, fie=fie, foe=foe, fum=fum"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not be (fumjavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))
        fumjavaMap should (not be (tojavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or (not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM"))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (fumjavaMap) or not contain oneOf (Entry("fee", "fee"), Entry("fie", "fie"), Entry("foe", "foe"), Entry("fum", "fum")))) (decided by upperCaseStringEquality)
        (fumjavaMap should (not be (tojavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) or not contain oneOf (Entry("FEE", "FEE"), Entry("FIE", "FIE"), Entry("FOE", "FOE"), Entry("FUM", "FUM")))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "FEE=FEE, FIE=FIE, FOE=FOE, FUM=FUM"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")) or not contain oneOf (Entry(" FEE ", " FEE "), Entry(" FIE ", " FIE "), Entry(" FOE ", " FOE "), Entry(" FUU ", " FUU ")))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(1, 1)))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap(Entry(2, 2)))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(Entry(1, 1)), javaMap(Entry(1, 1)), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")), javaMap(Entry("hi", "hi")))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap(Entry("to", "to")), javaMap(Entry("to", "to")), javaMap(Entry("to", "to")))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (contain oneOf (3, 2, 5) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (contain oneOf (Entry(3, 3), Entry(2, 2), Entry(1, 1)) or contain oneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        atLeast (2, lists) should (contain oneOf (Entry(3, 3), Entry(1, 1), Entry(5, 5)) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain oneOf (Entry(3, 3), Entry(6, 6), Entry(5, 5)) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        atLeast (2, lists) should (contain oneOf (Entry(3, 3), Entry(1, 1), Entry(5, 5)) or contain oneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (Entry(6, 6), Entry(7, 7), Entry(8, 8)) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain one of (6=6, 7=7, 8=8), and " + decorateToStringValue(javaMap(Entry(2, 2))) + " did not contain one of (1=1, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (contain oneOf (Entry("ho", "ho")) or contain oneOf (Entry("he", "he")))
        all (hijavaMaps) should (contain oneOf (Entry("hi", "hi")) or contain oneOf (Entry("he", "he")))
        all (hijavaMaps) should (contain oneOf (Entry("ho", "ho")) or contain oneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain oneOf (Entry("hi", "hi")) or contain oneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain oneOf (Entry("ho", "ho")) or contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (contain oneOf (Entry("hi", "hi")) or contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (contain oneOf (Entry("ho", "ho")) or contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain oneOf (Entry("hi", "hi")) or contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(Entry(1, 1))) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (be (javaMap(Entry(2, 2))) or contain oneOf (Entry(1, 1), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (be (javaMap(Entry(1, 1))) or contain oneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(Entry(2, 2))) or contain oneOf (Entry(2, 2), Entry(3, 3), Entry(8, 8)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was not equal to " + decorateToStringValue(javaMap(Entry(2, 2))) + ", and " + decorateToStringValue(javaMap(Entry(1, 1))) + " did not contain one of (2=2, 3=3, 8=8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain oneOf (Entry("he", "he")))
        all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain oneOf (Entry("he", "he")))
        all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain oneOf (Entry("hi", "hi")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain oneOf (Entry("hi", "hi")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (be (javaMap(Entry("hi", "hi"))) or contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap(Entry("ho", "ho"))) or contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was not equal to " + decorateToStringValue(javaMap(Entry("ho", "ho"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " did not contain one of (hi=hi)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain oneOf (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain oneOf (Entry(1, 1), Entry(2, 2), Entry(8, 8)) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not contain oneOf (Entry(3, 3), Entry(2, 2), Entry(8, 8)) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(1, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf (Entry(2, 2), Entry(6, 6), Entry(8, 8)) or not contain oneOf (Entry(2, 2), Entry(3, 3), Entry(4, 4)))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(Entry(2, 2))) + " contained one of (2=2, 6=6, 8=8), and " + decorateToStringValue(javaMap(Entry(2, 2))) + " contained one of (2=2, 3=3, 4=4)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not contain oneOf (Entry("hi", "hi")) or not contain oneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (not contain oneOf (Entry("ho", "ho")) or not contain oneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (not contain oneOf (Entry("hi", "hi")) or not contain oneOf (Entry("ho", "ho")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain oneOf (Entry("ho", "ho")) or not contain oneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain oneOf (Entry("hi", "hi")) or not contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (not contain oneOf (Entry("ho", "ho")) or not contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (not contain oneOf (Entry("hi", "hi")) or not contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain oneOf (Entry("ho", "ho")) or not contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho), and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaMap(Entry(1, 1))) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(4, 4)))
        all (list1s) should (not be (javaMap(Entry(2, 2))) or not contain oneOf (Entry(8, 8), Entry(3, 3), Entry(1, 1)))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaMap(Entry(1, 1))) or not contain oneOf (Entry(2, 2), Entry(3, 3), Entry(1, 1)))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry(1, 1))) + " was equal to " + decorateToStringValue(javaMap(Entry(1, 1))) + ", and " + decorateToStringValue(javaMap(Entry(1, 1))) + " contained one of (2=2, 3=3, 1=1)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain oneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain oneOf (Entry("hi", "hi")))
        all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain oneOf (Entry("ho", "ho")))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain oneOf (Entry("ho", "ho")))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain oneOf (Entry("hi", "hi")))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (not be (javaMap(Entry("ho", "ho"))) or not contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap(Entry("hi", "hi"))) or not contain oneOf (Entry("ho", "ho")))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(Entry("hi", "hi"))) + " was equal to " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + ", and " + decorateToStringValue(javaMap(Entry("hi", "hi"))) + " contained one of (ho=ho)", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
