/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class MapShouldContainNoneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a.map(e => (e._1.toUpperCase, e._2.toUpperCase)) == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainNoneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and contain noneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain noneOf ("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and contain noneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain noneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain noneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain noneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain noneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain noneOf("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          fumList should (not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe")))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ") and contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain noneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain noneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (3 -> 3, 2 -> 2, 8 -> 8) and contain noneOf (2 -> 2, 6 -> 6, 8 -> 8))
        atLeast (2, lists) should (contain noneOf (3, 2, 5) and contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain noneOf (3 -> 3, 2 -> 2, 8 -> 8) and contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain noneOf (1 -> 1, 2 -> 2, 8 -> 8) and contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2 -> 2, 6 -> 6, 8 -> 8) and contain noneOf (3 -> 3, 6 -> 6, 9 -> 9))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (3 -> 3, 6 -> 6, 9 -> 9) and contain noneOf (2 -> 2, 6 -> 6, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((3,3), (6,6), (9,9)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ("ho" -> "ho", "hello" -> "hello") and contain noneOf ("hi" -> "hi", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((ho,ho), (hello,hello)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((hi,hi), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (3 -> 3, 6 -> 6, 9 -> 9) and contain noneOf (2 -> 2, 6 -> 6, 8 -> 8))
        }
        checkMessageStackDepth(e4, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((3,3), (6,6), (9,9)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain noneOf ("hi" -> "hi") and contain noneOf ("he" -> "he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ("HI" -> "HI") and contain noneOf ("he" -> "he"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ("he" -> "he") and contain noneOf ("HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((he,he)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain noneOf ("hi" -> "hi") and contain noneOf ("he" -> "he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ("HI" -> "HI") and contain noneOf ("he" -> "he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ("he" -> "he") and contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((he,he)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (be (scala.collection.mutable.LinkedHashMap(8 -> 8)) and contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (3 -> 3, 6 -> 6, 9 -> 9))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " contained one of ((1,1), (2,2), (3,3))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain noneOf ("hi" -> "hi", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((hi,hi), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " contained one of ((1,1), (2,2), (3,3))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain noneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain noneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain noneOf ("HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain noneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and contain noneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain noneOf (1 -> 1, 2 -> 2, 8 -> 8) and not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        atLeast (2, lists) should (not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8) and not contain noneOf (1 -> 1, 3 -> 3, 8 -> 8))
        atMost (2, lists) should (not contain noneOf (1 -> 1, 3 -> 3, 8 -> 8) and contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        no (lists) should (not contain noneOf (3 -> 3, 6 -> 6, 9 -> 9) and not contain noneOf (5 -> 5, 7 -> 7, 9 -> 9))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8) and not contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (1 -> 1, 2 -> 2, 3 -> 3) and not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((1,1), (2,2), (3,3)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ("ho" -> "ho", "hello" -> "hello") and not contain noneOf ("hi" -> "hi", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((ho,ho), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ("hi" -> "hi", "hey" -> "hey", "howdy" -> "howdy") and not contain noneOf ("ho" -> "ho", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((hi,hi), (hey,hey), (howdy,howdy)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((ho,ho), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain noneOf ("HI" -> "HI") and not contain noneOf ("HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ("hi" -> "hi") and not contain noneOf ("HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ("HI" -> "HI") and not contain noneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain noneOf ("HI" -> "HI") and not contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain noneOf ("hi" -> "hi") and not contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain noneOf ("HI" -> "HI") and not contain noneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((HI,HI)), but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        atMost (2, lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        no (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) and not contain noneOf (3 -> 3, 6 -> 6, 9 -> 9))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) and not contain noneOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (scala.collection.mutable.LinkedHashMap(3 -> 3)) and not contain noneOf (1 -> 1, 6 -> 6, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(3 -> 3)) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (6,6), (8,8))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain noneOf ("hi" -> "hi", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain noneOf ("ho" -> "ho", "hello" -> "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((ho,ho), (hello,hello))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain noneOf ("HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain noneOf ("HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain noneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) and not contain noneOf ("HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) and not contain noneOf ("hi" -> "hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", but " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
