/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class MapShouldContainOneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a.map(e => (e._1.toUpperCase, e._2.toUpperCase)) == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainOneOfLogicalOrSpec.scala"
  
  object `a List` {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")
    
    object `when used with (contain oneOf (..) or contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain oneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (equal (toList) or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (equal (fumList) or contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (equal (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (equal (fumList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (legacyEqual (toList) or contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (legacyEqual (fumList) or contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (legacyEqual (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (legacyEqual (fumList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (fumList))
        fumList should (contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (fumList))
        fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(fie,fie), (fee,fee), (fum,fum), (foe,foe)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not equal (fumList) or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not equal (toList) or not contain oneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not equal (fumList) or not contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (not equal (toList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) or not contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain oneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not be (fumList) or not contain oneOf("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        fumList should (not be (toList) or not contain oneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        fumList should (not be (fumList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        fumList should (not be (toList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain oneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain oneOf (3, 2, 5) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain oneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain oneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        atLeast (2, lists) should (contain oneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain oneOf (3 -> 3, 6 -> 6, 5 -> 5) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain oneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain oneOf (8 -> 8, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (6 -> 6, 7 -> 7, 8 -> 8) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((6,6), (7,7), (8,8)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain oneOf ("ho" -> "ho") or contain oneOf ("he" -> "he"))
        all (hiLists) should (contain oneOf ("hi" -> "hi") or contain oneOf ("he" -> "he"))
        all (hiLists) should (contain oneOf ("ho" -> "ho") or contain oneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ("hi" -> "hi") or contain oneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain oneOf ("ho" -> "ho") or contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain oneOf ("hi" -> "hi") or contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain oneOf ("ho" -> "ho") or contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ("hi" -> "hi") or contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or contain oneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or contain oneOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " did not contain one of ((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain oneOf ("he" -> "he"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain oneOf ("he" -> "he"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain oneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain oneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain oneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain oneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain oneOf (1 -> 1, 2 -> 2, 8 -> 8) or not contain oneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain oneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain oneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf (2 -> 2, 6 -> 6, 8 -> 8) or not contain oneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (6,6), (8,8)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain oneOf ("hi" -> "hi") or not contain oneOf ("hi" -> "hi"))
        all (hiLists) should (not contain oneOf ("ho" -> "ho") or not contain oneOf ("hi" -> "hi"))
        all (hiLists) should (not contain oneOf ("hi" -> "hi") or not contain oneOf ("ho" -> "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ("ho" -> "ho") or not contain oneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain oneOf ("hi" -> "hi") or not contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain oneOf ("ho" -> "ho") or not contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain oneOf ("hi" -> "hi") or not contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain oneOf ("ho" -> "ho") or not contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or not contain oneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or not contain oneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or not contain oneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or not contain oneOf (2 -> 2, 3 -> 3, 1 -> 1))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " contained one of ((2,2), (3,3), (1,1))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain oneOf ("hi" -> "hi"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain oneOf ("hi" -> "hi"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain oneOf ("ho" -> "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain oneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain oneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain oneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
