/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainOneOfSpec extends Spec with Matchers {

  val upperCaseEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }

  object `a List` {

    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")
    val fumfuList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum", "fu" -> "fu")

    object `when used with contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumList should contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
        // Here it contains two of, not one of
        val e2 = intercept[TestFailedException] {
          fumfuList should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")
        }
        e2.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumfuList), "(fee,fee), (fum,fum), (foe,foe), (fu,fu)"))
        // Contains duplicate elements in the right list
        val e3 = intercept[IllegalArgumentException] {
          fumList should contain oneOf ("fee", "fum", "foe", "fum")
        }
        e3.getMessage should be (Resources("oneOfDuplicate", "\"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumList should contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")
        intercept[TestFailedException] {
          fumList should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumList should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          fumList should contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
        (fumList should contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumList should (contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))
        intercept[TestFailedException] {
          fumList should (contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumList should (contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu"))) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        }
        (fumList should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

/*
 I purposely don't want to support this syntax:

        fumList should contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        fumList should (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))

 Reason is that I don't want people putting parentheses between contain and oneOf, etc. This will not compile.
*/
    object `when used with not contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          toList should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(toList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        toList should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        intercept[TestFailedException] {
          toList should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (toList should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        toList should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        intercept[TestFailedException] {
          (toList should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the top one does happen to compile and run:

        toList should not contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        // toList should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        // toList should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))

The bottom two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          toList should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(toList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        toList should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        intercept[TestFailedException] {
          toList should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (toList should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        }
        toList should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (toList should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a collection of Lists` {

    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))

    object `when used with contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        atLeast (2, lists) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        atMost (2, lists) should contain oneOf (2 -> 2, 3 -> 3, 4 -> 4)
        no (lists) should contain oneOf (3 -> 3, 4 -> 4, 5 -> 5)
        no (nils) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        no (listsMap) should contain oneOf (3 -> 3, 4 -> 4, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (3,3), (4,4)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should contain oneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy")
        }
        e2.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain one of ((ho,ho), (hey,hey), (howdy,howdy)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        }
        e4.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain one of ((1,1), (3,3), (4,4)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain oneOf ("hi" -> "hi")
        intercept[TestFailedException] {
          all (hiLists) should contain oneOf ("ho" -> "ho")
        }
        implicit val ise = upperCaseEquality
        all (hiLists) should contain oneOf ("HI" -> "HI")
        intercept[TestFailedException] {
          all (hiLists) should contain oneOf ("hi" -> "hi")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain oneOf ("HI" -> "HI")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain oneOf ("hi" -> "hi")) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiLists) should contain oneOf ("hi" -> "hi")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should contain oneOf ("ho" -> "ho")) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with (contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain oneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain oneOf (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (contain oneOf (3 -> 3, 4 -> 4, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain one of ((1,1), (3,3), (4,4)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should (contain oneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        e2.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain one of ((ho,ho), (hey,hey), (howdy,howdy)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e4.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap()) + " did not contain one of ((1,1), (3,3), (4,4)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain oneOf ("hi" -> "hi"))
        intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ("ho" -> "ho"))
        }
        implicit val ise = upperCaseEquality
        all (hiLists) should (contain oneOf ("HI" -> "HI"))
        intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ("hi" -> "hi"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain oneOf ("HI" -> "HI"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ("hi" -> "hi"))) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiLists) should (contain oneOf ("hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ("ho" -> "ho"))) (decided by defaultEquality[(String, String)])
        }
      }
    }

/*
 I purposely don't want to support this syntax:

scala> all (list1s) should contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4))
                                             ^

scala> all (list1s) should (contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4)))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should (contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4)))
                                              ^

 Reason is that I don't want people putting parentheses between contain and oneOf, etc. This will not compile.
*/
    object `when used with not contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to")) + " contained one of ((happy,happy), (birthday,birthday), (to,to), (you,you)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toLists) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        intercept[TestFailedException] {
          all (toLists) should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        all (toLists) should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        intercept[TestFailedException] {
          (all (toLists) should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the last one does happen to compile and run:

scala> all (toLists) should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toLists) should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))
                                                    ^

scala> all (toLists) should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toLists) should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
                                                   ^

scala> all (toLists) should not contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))

The top two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("to" -> "to")) + " contained one of ((happy,happy), (birthday,birthday), (to,to), (you,you)) (MapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toLists) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        intercept[TestFailedException] {
          all (toLists) should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        }
        all (toLists) should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (all (toLists) should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
