/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainAtLeastOneOfSpec extends Spec with Matchers {

  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }

  object `a List` {

    val fumList: String = "u"
    val toList: String = "o"

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain atLeastOneOf ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          fumList should contain atLeastOneOf ('h', 'b', 'o', 'y')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should contain atLeastOneOf ('h', 'b', 'o', 'y')
        intercept[TestFailedException] {
          fumList should contain atLeastOneOf ('u', 'u', 'u', 'u')
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain atLeastOneOf ('h', 'b', 'o', 'y')) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumList should contain atLeastOneOf ('u', 'u', 'u', 'u')) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumList should contain atLeastOneOf ('E', 'I', 'P', 'U')
        }
        (fumList should contain atLeastOneOf ('E', 'I', 'P', 'U')) (after being lowerCased)
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain atLeastOneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y'))
        intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('u', 'u', 'u', 'u'))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ('u', 'u', 'u', 'u'))) (decided by invertedStringEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('E', 'I', 'P', 'U'))
        }
        (fumList should (contain atLeastOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
    }

/*
 I purposely don't want to support this syntax:

        fumList should contain (atLeastOneOf ('e', 'i', 'p', 'u'))
        fumList should (contain (atLeastOneOf ('e', 'i', 'p', 'u')))

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain atLeastOneOf ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          toList should not contain atLeastOneOf ('h', 'b', 'o', 'y')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(toList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        toList should not contain atLeastOneOf ('o', 'o', 'o', 'o')
        intercept[TestFailedException] {
          toList should not contain atLeastOneOf ('e', 'i', 'p', 'u')
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain atLeastOneOf ('o', 'o', 'o', 'o')) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (toList should not contain atLeastOneOf ('e', 'i', 'p', 'u')) (decided by invertedStringEquality)
        }
        toList should not contain atLeastOneOf ('O', 'O', 'O', 'O')
        intercept[TestFailedException] {
          (toList should not contain atLeastOneOf ('O', 'O', 'O', 'O')) (after being lowerCased)
        }
      }
    }

/*
Interesting, of these three, the top one does happen to compile and run:

        toList should not contain (atLeastOneOf ('e', 'i', 'p', 'u'))
        // toList should not (contain (atLeastOneOf ('e', 'i', 'p', 'u')))
        // toList should (not (contain (atLeastOneOf ('e', 'i', 'p', 'u'))))

The bottom two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          toList should (not contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAtLeastOneOf", decorateToStringValue(toList), "'h', 'b', 'o', 'y'"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        toList should (not contain atLeastOneOf ('o', 'o', 'o', 'o'))
        intercept[TestFailedException] {
          toList should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain atLeastOneOf ('o', 'o', 'o', 'o'))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (toList should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))) (decided by invertedStringEquality)
        }
        toList should (not contain atLeastOneOf ('O', 'O', 'O', 'O'))
        intercept[TestFailedException] {
          (toList should (not contain atLeastOneOf ('O', 'O', 'O', 'O'))) (after being lowerCased)
        }
      }
    }
  }

  object `a collection of Lists` {

    val list1s: Vector[String] = Vector("1", "1", "1")
    val lists: Vector[String] = Vector("1", "1", "2")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("i", "i", "i")
    val toLists: Vector[String] = Vector("o", "o", "o")

    object `when used with contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain atLeastOneOf ('1', '3', '4')
        atLeast (2, lists) should contain atLeastOneOf ('1', '3', '4')
        atMost (2, lists) should contain atLeastOneOf ('2', '3', '4')
        no (lists) should contain atLeastOneOf ('3', '4', '5')
        no (nils) should contain atLeastOneOf ('1', '3', '4')
        no (listsString) should contain atLeastOneOf ('3', '4', '5')

        val e1 = intercept[TestFailedException] {
          all (lists) should contain atLeastOneOf ('1', '3', '4')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("2") + " did not contain at least one of ('1', '3', '4') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should contain atLeastOneOf ('o', 'e', 'd')
        }
        e2.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("") + " did not contain at least one of ('o', 'e', 'd') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsString) should contain atLeastOneOf ('1', '3', '4')
        }
        e4.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("") + " did not contain at least one of ('1', '3', '4') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsString)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain atLeastOneOf ('i')
        intercept[TestFailedException] {
          all (hiLists) should contain atLeastOneOf ('o')
        }
        implicit val ise = invertedStringEquality
        all (hiLists) should contain atLeastOneOf ('o')
        intercept[TestFailedException] {
          all (hiLists) should contain atLeastOneOf ('i')
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain atLeastOneOf ('o')) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain atLeastOneOf ('i')) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hiLists) should contain atLeastOneOf ('i')) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should contain atLeastOneOf ('o')) (decided by defaultEquality[Char])
        }
      }
    }

    object `when used with (contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain atLeastOneOf ('1', '3', '4'))
        atMost (2, lists) should (contain atLeastOneOf ('2', '3', '4'))
        no (lists) should (contain atLeastOneOf ('3', '4', '5'))
        no (nils) should (contain atLeastOneOf ('1', '3', '4'))
        no (listsString) should (contain atLeastOneOf ('3', '4', '5'))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf ('1', '3', '4'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("2") + " did not contain at least one of ('1', '3', '4') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf ('o', 'e', 'd'))
        }
        e2.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("") + " did not contain at least one of ('o', 'e', 'd') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsString) should (contain atLeastOneOf ('1', '3', '4'))
        }
        e4.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("") + " did not contain at least one of ('1', '3', '4') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsString)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain atLeastOneOf ('i'))
        intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ('o'))
        }
        implicit val ise = invertedStringEquality
        all (hiLists) should (contain atLeastOneOf ('o'))
        intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ('i'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atLeastOneOf ('o'))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ('i'))) (decided by invertedStringEquality)
        }
        implicit val ise = invertedStringEquality
        (all (hiLists) should (contain atLeastOneOf ('i'))) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ('o'))) (decided by defaultEquality[Char])
        }
      }
    }

/*
 I purposely don't want to support this syntax:

scala> all (list1s) should contain (atLeastOneOf ('1', '3', '4'))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should contain (atLeastOneOf ('1', '3', '4'))
                                             ^

scala> all (list1s) should (contain (atLeastOneOf ('1', '3', '4')))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should (contain (atLeastOneOf ('1', '3', '4')))
                                              ^

 Reason is that I don't want people putting parentheses between contain and atLeastOneOf, etc. This will not compile.
*/
    object `when used with not contain atLeastOneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain atLeastOneOf ('e', 'i', 'p', 'u')
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain atLeastOneOf ('h', 'b', 'o', 'y')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("o") + " contained at least one of ('h', 'b', 'o', 'y') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (toLists) should not contain atLeastOneOf ('o', 'o', 'o', 'o')
        intercept[TestFailedException] {
          all (toLists) should not contain atLeastOneOf ('e', 'i', 'p', 'u')
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain atLeastOneOf ('o', 'o', 'o', 'o')) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain atLeastOneOf ('e', 'i', 'p', 'u')) (decided by invertedStringEquality)
        }
        all (toLists) should not contain atLeastOneOf ('O', 'O', 'O', 'O')
        intercept[TestFailedException] {
          (all (toLists) should not contain atLeastOneOf ('O', 'O', 'O', 'O')) (after being lowerCased)
        }
      }
    }

/*
Interesting, of these three, the last one does happen to compile and run:

scala> all (toLists) should (not (contain (atLeastOneOf ('e', 'i', 'p', 'u'))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toLists) should (not (contain (atLeastOneOf ('e', 'i', 'p', 'u'))))
                                                    ^

scala> all (toLists) should not (contain (atLeastOneOf ('e', 'i', 'p', 'u')))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (toLists) should not (contain (atLeastOneOf ('e', 'i', 'p', 'u')))
                                                   ^

scala> all (toLists) should not contain (atLeastOneOf ('e', 'i', 'p', 'u'))

The top two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain atLeastOneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainAtLeastOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("o") + " contained at least one of ('h', 'b', 'o', 'y') (StringShouldContainAtLeastOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        all (toLists) should (not contain atLeastOneOf ('o', 'o', 'o', 'o'))
        intercept[TestFailedException] {
          all (toLists) should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain atLeastOneOf ('o', 'o', 'o', 'o'))) (decided by invertedStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain atLeastOneOf ('e', 'i', 'p', 'u'))) (decided by invertedStringEquality)
        }
        all (toLists) should (not contain atLeastOneOf ('O', 'O', 'O', 'O'))
        intercept[TestFailedException] {
          (all (toLists) should (not contain atLeastOneOf ('O', 'O', 'O', 'O'))) (after being lowerCased)
        }
      }
    }
  }
}
