/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class StringShouldContainNoneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a.toString.toUpperCase.toCharArray()(0) == b
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainNoneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: String = "u"
    val toList: String = "o"
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain noneOf ('e', 'i', 'p', 'a') and contain noneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ('e', 'i', 'p', 'u') and contain noneOf ('i', 'e', 'a', 'p'))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ('e', 'i', 'p', 'a') and contain noneOf ('i', 'e', 'u', 'p'))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain noneOf ('e', 'i', 'p', 'u') and contain noneOf ('e', 'i', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ('E', 'I', 'P', 'U') and contain noneOf ('e', 'i', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ('e', 'i', 'p', 'u') and (contain noneOf ('E', 'I', 'U', 'P')))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain noneOf ('e', 'i', 'p', 'u') and contain noneOf ('e', 'i', 'u', 'p'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain noneOf ('E', 'I', 'P', 'U') and contain noneOf ('e', 'i', 'u', 'p'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain noneOf ('e', 'i', 'p', 'u') and contain noneOf ('E', 'I', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf ('E', 'I', 'P', 'A') and contain noneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (equal (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain noneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain noneOf ('e', 'i', 'p', 'a'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain noneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain noneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain noneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain noneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain noneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain noneOf ('E', 'I', 'P', 'A'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    object `when used with (legacyEqual (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain noneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain noneOf ('e', 'i', 'p', 'a'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain noneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain noneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain noneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain noneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain noneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain noneOf ('E', 'I', 'P', 'A'))) (after being lowerCased)
      }
    }

    object `when used with (contain noneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain noneOf('i', 'e', 'a', 'p') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ('e', 'i', 'p', 'a') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ('e', 'i', 'p', 'u') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain noneOf ('e', 'i', 'p', 'u') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain noneOf ('E', 'I', 'P', 'U') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain noneOf ('E', 'I', 'P', 'U') and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain noneOf ('e', 'i', 'p', 'u') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain noneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain noneOf ('e', 'i', 'p', 'u') and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf ('E', 'I', 'P', 'A') and legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain noneOf ('e', 'i', 'p', 'u') and not contain noneOf('e', 'i', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('e', 'i', 'u', 'p'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain noneOf ('e', 'i', 'p', 'u') and not contain noneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('E', 'I', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain noneOf ('e', 'i', 'p', 'u') and not contain noneOf ('E', 'I', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          fumList should (not contain noneOf ('E', 'I', 'P', 'U') and (not contain noneOf ('e', 'i', 'u', 'p')))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('E', 'I', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain noneOf ('e', 'i', 'p', 'u') and not contain noneOf ('E', 'I', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('e', 'i', 'u', 'p'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'u', 'p'"), fileName, thisLineNumber - 2)
        (fumList should (contain noneOf ('E', 'I', 'P', 'A') and contain noneOf ('E', 'I', 'P', 'A'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not equal (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain noneOf('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain noneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain noneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain noneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain noneOf ('e', 'i', 'p', 'u')))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain noneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain noneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain noneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain noneOf('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain noneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain noneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain noneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain noneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain noneOf ('e', 'i', 'p', 'u')))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain noneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain noneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain noneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain noneOf ('E', 'I', 'P', 'U') and not contain noneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("1", "1", "1")
    val lists: Vector[String] = Vector("1", "1", "2")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("i", "i", "i")
    val toLists: Vector[String] = Vector("o", "o", "o")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf ('3', '2', '8') and contain noneOf ('2', '6', '8'))
        atLeast (2, lists) should (contain noneOf (3, 2, 5) and contain noneOf ('2', '3', '4'))
        atMost (2, lists) should (contain noneOf ('3', '2', '8') and contain noneOf ('2', '3', '4'))
        no (lists) should (contain noneOf ('1', '2', '8') and contain noneOf ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf ('2', '6', '8') and contain noneOf ('3', '6', '9'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " contained one of ('2', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain noneOf ('3', '6', '9') and contain noneOf ('2', '6', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " did not contain one of ('3', '6', '9'), but " + decorateToStringValue("2") + " contained one of ('2', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ('o', 'l') and contain noneOf ('i', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('o', 'l'), but " + decorateToStringValue("i") + " contained one of ('i', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (lists) should (contain noneOf ('3', '6', '9') and contain noneOf ('2', '6', '8'))
        }
        checkMessageStackDepth(e4, allErrMsg(2, decorateToStringValue("2") + " did not contain one of ('3', '6', '9'), but " + decorateToStringValue("2") + " contained one of ('2', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain noneOf ('i') and contain noneOf ('e'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ('I') and contain noneOf ('e'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain noneOf ('e') and contain noneOf ('I'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('e'), but " + decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain noneOf ('i') and contain noneOf ('e'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ('I') and contain noneOf ('e'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain noneOf ('e') and contain noneOf ('I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('e'), but " + decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("1") and contain noneOf ('2', '3', '4'))
        atLeast (2, lists) should (be ("1") and contain noneOf ('2', '3', '4'))
        atMost (2, lists) should (be ("1") and contain noneOf ('2', '3', '4'))
        no (lists) should (be ("8") and contain noneOf ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be ("1") and contain noneOf ('3', '6', '9'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[1]"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain noneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " contained one of ('1', '2', '3')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain noneOf ('i', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " contained one of ('i', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain noneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " contained one of ('1', '2', '3')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("i") and contain noneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be ("o") and contain noneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain noneOf ('I'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("i") and contain noneOf ('i'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be ("o") and contain noneOf ('i'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be ("i") and contain noneOf ('I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " contained one of ('I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain noneOf ('1', '2', '8') and not contain noneOf ('1', '6', '8'))
        atLeast (2, lists) should (not contain noneOf ('1', '6', '8') and not contain noneOf ('1', '3', '8'))
        atMost (2, lists) should (not contain noneOf ('1', '3', '8') and contain noneOf ('1', '6', '8'))
        no (lists) should (not contain noneOf ('3', '6', '9') and not contain noneOf ('5', '7', '9'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf ('1', '6', '8') and not contain noneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " did not contain one of ('1', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf ('1', '2', '3') and not contain noneOf ('1', '6', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " contained one of ('1', '2', '3'), but " + decorateToStringValue("2") + " did not contain one of ('1', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ('o', 'l') and not contain noneOf ('i', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('o', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ('i', 'e', 'd') and not contain noneOf ('o', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " contained one of ('i', 'e', 'd'), but " + decorateToStringValue("i") + " did not contain one of ('o', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain noneOf ('I') and not contain noneOf ('I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ('i') and not contain noneOf ('I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain noneOf ('I') and not contain noneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained one of ('I'), but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain noneOf ('I') and not contain noneOf ('I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain noneOf ('i') and not contain noneOf ('I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain noneOf ('I') and not contain noneOf ('i'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained one of ('I'), but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") and not contain noneOf ('1', '2', '3'))
        atLeast (2, lists) should (not be ("3") and not contain noneOf ('1', '6', '8'))
        atMost (2, lists) should (not be ("3") and contain noneOf ('1', '6', '8'))
        no (list1s) should (not be ("1") and not contain noneOf ('3', '6', '9'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be ("2") and not contain noneOf ('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " was equal to " + decorateToStringValue("2"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be ("3") and not contain noneOf ('1', '6', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[3]") + ", but " + decorateToStringValue("2") + " did not contain one of ('1', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain noneOf ('i', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain noneOf ('o', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " did not contain one of ('o', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be ("o") and not contain noneOf ('I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain noneOf ('I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain noneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be ("o") and not contain noneOf ('I'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("i") and not contain noneOf ('I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("o") and not contain noneOf ('i'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
