/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class StringShouldContainOneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a.toString.toUpperCase.toCharArray()(0) == b
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainOneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: String = "u"
    val toList: String = "o"
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf ('e', 'i', 'p', 'u') and contain oneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('h', 'b', 'o', 'y') and contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain oneOf ('e', 'i', 'p', 'u') and contain oneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ('E', 'I', 'P', 'U') and contain oneOf ('I', 'E', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('e', 'i', 'p', 'u') and contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain oneOf ('E', 'I', 'P', 'U') and (contain oneOf ('e', 'i', 'p', 'u')))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') and contain oneOf ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('e', 'i', 'p', 'u') and contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('E', 'I', 'P', 'U') and contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') and contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain oneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain oneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain oneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain oneOf ('e', 'i', 'p', 'u')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain oneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain oneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain oneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain oneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain oneOf ('e', 'i', 'p', 'u')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf('i', 'e', 'u', 'p') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('e', 'i', 'p', 'u') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain oneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('i', 'e', 'u', 'p') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain oneOf ('i', 'e', 'u', 'p') and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('i', 'e', 'u', 'p') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('E', 'I', 'P', 'U') and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain oneOf ('E', 'I', 'P', 'U') and not contain oneOf('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('e', 'i', 'p', 'u') and not contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('E', 'I', 'P', 'U') and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain oneOf ('e', 'i', 'p', 'u') and not contain oneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('E', 'I', 'P', 'U') and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        
        val e2 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('e', 'i', 'p', 'u') and (not contain oneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain oneOf ('e', 'i', 'p', 'u') and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain oneOf ('E', 'I', 'P', 'U') and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain oneOf ('e', 'i', 'p', 'u') and not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') and contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain oneOf('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain oneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain oneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf ('E', 'I', 'P', 'D') and not contain oneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain oneOf('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain oneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain oneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf ('E', 'I', 'P', 'D') and not contain oneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("1", "1", "1")
    val lists: Vector[String] = Vector("1", "1", "2")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("i", "i", "i")
    val toLists: Vector[String] = Vector("o", "o", "o")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf ('3', '2', '1') and contain oneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain oneOf ('3', '1', '5') and contain oneOf ('1', '3', '4'))
        atMost (2, lists) should (contain oneOf ('3', '2', '8') and contain oneOf ('2', '3', '4'))
        no (lists) should (contain oneOf ('3', '6', '9') and contain oneOf ('3', '4', '5'))
        no (nils) should (contain oneOf ('1', '2', '8') and contain oneOf ('1', '3', '4'))
        no (listsString) should (contain oneOf('3', '8', '5') and contain oneOf ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf ('1', '6', '8') and contain oneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " did not contain one of ('1', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain oneOf ('1', '2', '8') and contain oneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " contained one of ('1', '2', '8'), but " + decorateToStringValue("2") + " did not contain one of ('1', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain oneOf ('i', 'l') and contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("") + " did not contain one of ('i', 'l')", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ('i', 'l') and contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " contained one of ('i', 'l'), but " + decorateToStringValue("i") + " did not contain one of ('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsString) should (contain oneOf ('1', '3', '4') and contain oneOf ('1', '3'))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue("") + " did not contain one of ('1', '3', '4')", thisLineNumber - 2, listsString), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain oneOf ('1', '2', '8') and contain oneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue("2") + " contained one of ('1', '2', '8'), but " + decorateToStringValue("2") + " did not contain one of ('1', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain oneOf ('o') and contain oneOf ('e'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ('i') and contain oneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ('o') and contain oneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o'), but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain oneOf ('o') and contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ('i') and contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ('o') and contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o'), but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("1") and contain oneOf ('1', '3', '4'))
        atLeast (2, lists) should (be ("1") and contain oneOf ('1', '3', '4'))
        atMost (2, lists) should (be ("1") and contain oneOf ('2', '3', '4'))
        no (lists) should (be ("8") and contain oneOf ('3', '4', '5'))
        no (nils) should (be ("8") and contain oneOf ('1', '3', '4'))
        no (listsString) should (be ("8") and contain oneOf ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be ("1") and contain oneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[1]"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain oneOf ('2', '3', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " did not contain one of ('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be ("e") and contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[e]"), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain one of ('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsString) should (be ("1") and contain oneOf ('1', '3'))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[1]"), thisLineNumber - 2, listsString), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain oneOf ('2', '3', '8'))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " did not contain one of ('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be ("i") and contain oneOf ('e'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be ("o") and contain oneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain oneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("i") and contain oneOf ('o'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be ("o") and contain oneOf ('o'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be ("i") and contain oneOf ('i'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain oneOf ('3', '2', '8') and not contain oneOf ('8', '3', '4'))
        atLeast (2, lists) should (not contain oneOf ('3', '8', '5') and not contain oneOf ('8', '3', '4'))
        atMost (2, lists) should (not contain oneOf ('3', '6', '8') and contain oneOf ('5', '3', '4'))
        no (lists) should (not contain oneOf ('1', '2', '9') and not contain oneOf ('2', '1', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf ('2', '6', '8') and not contain oneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " contained one of ('2', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf ('3', '6', '8') and not contain oneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " did not contain one of ('3', '6', '8'), but " + decorateToStringValue("2") + " contained one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ('i', 'l') and not contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " contained one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ('o', 'e', 'd') and not contain oneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('o', 'e', 'd'), but " + decorateToStringValue("i") + " contained one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain oneOf ('i') and not contain oneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ('o') and not contain oneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ('i') and not contain oneOf ('o'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i'), but " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain oneOf ('i') and not contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain oneOf ('o') and not contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain oneOf ('i') and not contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i'), but " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") and not contain oneOf ('8', '3', '4'))
        atLeast (2, lists) should (not be ("3") and not contain oneOf ('8', '3', '4'))
        atMost (2, lists) should (not be ("3") and contain oneOf ('5', '3', '4'))
        no (list1s) should (not be ("1") and not contain oneOf ('2', '1', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be ("2") and not contain oneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " was equal to " + decorateToStringValue("2"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be ("3") and not contain oneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[3]") + ", but " + decorateToStringValue("2") + " contained one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain oneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain oneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be ("o") and not contain oneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain oneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain oneOf ('o'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be ("o") and not contain oneOf ('i'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("i") and not contain oneOf ('i'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("o") and not contain oneOf ('o'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
