/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class StringShouldContainOneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a.toString.toUpperCase.toCharArray()(0) == b
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainOneOfLogicalOrSpec.scala"
  
  object `a List` {
    
    val fumList: String = "u"
    val toList: String = "o"
    
    object `when used with (contain oneOf (..) or contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf ('e', 'i', 'p', 'u') or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (contain oneOf ('e', 'i', 'p', 'a') or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (contain oneOf ('e', 'i', 'p', 'u') or contain oneOf('i', 'e', 'a', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('e', 'i', 'p', 'a') or contain oneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'a'") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('i', 'e', 'u', 'p'))
        fumList should (contain oneOf ('i', 'e', 'u', 'p') or contain oneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('i', 'e', 'u', 'p') or (contain oneOf ('i', 'e', 'u', 'p')))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('i', 'e', 'u', 'p'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain oneOf ('i', 'e', 'u', 'p') or contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('i', 'e', 'u', 'p') or contain oneOf ('i', 'e', 'u', 'p'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (equal (toList) or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (equal (fumList) or contain oneOf('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (equal (toList) or contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (equal (fumList) or contain oneOf ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain oneOf ('i', 'e', 'u', 'p')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) or contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain oneOf ('i', 'e', 'u', 'p'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain oneOf ('i', 'e', 'u', 'p'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (legacyEqual (toList) or contain oneOf('i', 'e', 'u', 'p'))
        fumList should (legacyEqual (fumList) or contain oneOf('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (legacyEqual (toList) or contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (legacyEqual (fumList) or contain oneOf ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain oneOf ('i', 'e', 'u', 'p')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) or contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain oneOf ('i', 'e', 'u', 'p'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain oneOf ('i', 'e', 'u', 'p'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain oneOf('i', 'e', 'u', 'p') or legacyEqual (fumList))
        fumList should (contain oneOf('E', 'I', 'P', 'U') or legacyEqual (fumList))
        fumList should (contain oneOf('i', 'e', 'u', 'p') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (fumList))
        fumList should (contain oneOf ('i', 'e', 'u', 'p') or legacyEqual (fumList))
        fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain oneOf ('i', 'e', 'u', 'p') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain oneOf ('i', 'e', 'u', 'p') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain oneOf ('i', 'e', 'u', 'p') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumList), "'i', 'e', 'u', 'p'") + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf ('e', 'i', 'p', 'u'))
        fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf ('E', 'I', 'P', 'U'))
        fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf ('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain oneOf ('e', 'i', 'p', 'u') or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain oneOf ('E', 'I', 'P', 'U') or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (contain oneOf ('E', 'I', 'P', 'U') or contain oneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not equal (fumList) or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not equal (toList) or not contain oneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain oneOf ('i', 'e', 'u', 'p'))
        fumList should (not equal (fumList) or not contain oneOf ('i', 'e', 'u', 'p'))
        fumList should (not equal (toList) or not contain oneOf ('E', 'I', 'P', 'U'))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain oneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) or not contain oneOf ('i', 'e', 'u', 'p'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain oneOf ('i', 'e', 'u', 'p'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf ('E', 'I', 'P', 'D') or not contain oneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not be (fumList) or not contain oneOf('E', 'I', 'P', 'U'))
        fumList should (not be (toList) or not contain oneOf('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain oneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain oneOf ('e', 'i', 'p', 'u'))
        fumList should (not be (fumList) or not contain oneOf ('e', 'i', 'p', 'u'))
        fumList should (not be (toList) or not contain oneOf ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain oneOf ('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) or not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain oneOf ('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain oneOf ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'"), fileName, thisLineNumber - 2)
        (fumList should (not contain oneOf ('E', 'I', 'P', 'D') or not contain oneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("1", "1", "1")
    val lists: Vector[String] = Vector("1", "1", "2")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("i", "i", "i")
    val toLists: Vector[String] = Vector("o", "o", "o")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf ('3', '2', '1') or contain oneOf ('1', '3', '4'))
        all (list1s) should (contain oneOf (3, 2, 5) or contain oneOf ('1', '3', '4'))
        all (list1s) should (contain oneOf ('3', '2', '1') or contain oneOf ('2', '3', '4'))
        
        atLeast (2, lists) should (contain oneOf ('3', '1', '5') or contain oneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain oneOf ('3', '6', '5') or contain oneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain oneOf ('3', '1', '5') or contain oneOf ('8', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (6, 7, 8) or contain oneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " did not contain one of (6, 7, 8), and " + decorateToStringValue("2") + " did not contain one of ('1', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain oneOf ('o') or contain oneOf ('e'))
        all (hiLists) should (contain oneOf ('i') or contain oneOf ('e'))
        all (hiLists) should (contain oneOf ('o') or contain oneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain oneOf ('i') or contain oneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i'), and " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain oneOf ('o') or contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain oneOf ('i') or contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain oneOf ('o') or contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain oneOf ('i') or contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain one of ('i'), and " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("1") or contain oneOf ('1', '3', '4'))
        all (list1s) should (be ("2") or contain oneOf ('1', '3', '4'))
        all (list1s) should (be ("1") or contain oneOf ('2', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be ("2") or contain oneOf ('2', '3', '8'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[1]") + " was not equal to " + decorateToStringValue("[2]") + ", and " + decorateToStringValue("1") + " did not contain one of ('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be ("i") or contain oneOf ('e'))
        all (hiLists) should (be ("o") or contain oneOf ('e'))
        all (hiLists) should (be ("i") or contain oneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be ("o") or contain oneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", and " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("i") or contain oneOf ('o'))) (decided by invertedStringEquality)
        (all (hiLists) should (be ("o") or contain oneOf ('o'))) (decided by invertedStringEquality)
        (all (hiLists) should (be ("i") or contain oneOf ('i'))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be ("o") or contain oneOf ('i'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", and " + decorateToStringValue("i") + " did not contain one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain oneOf ('3', '2', '8') or not contain oneOf ('8', '3', '4'))
        all (list1s) should (not contain oneOf ('1', '2', '8') or not contain oneOf ('8', '3', '4'))
        all (list1s) should (not contain oneOf ('3', '2', '8') or not contain oneOf (8, 3, 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain oneOf ('2', '6', '8') or not contain oneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " contained one of ('2', '6', '8'), and " + decorateToStringValue("2") + " contained one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain oneOf ('i') or not contain oneOf ('i'))
        all (hiLists) should (not contain oneOf ('o') or not contain oneOf ('i'))
        all (hiLists) should (not contain oneOf ('i') or not contain oneOf ('o'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain oneOf ('o') or not contain oneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o'), and " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain oneOf ('i') or not contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain oneOf ('o') or not contain oneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain oneOf ('i') or not contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain oneOf ('o') or not contain oneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained one of ('o'), and " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") or not contain oneOf ('8', '3', '4'))
        all (list1s) should (not be ("1") or not contain oneOf ('8', '3', '4'))
        all (list1s) should (not be ("2") or not contain oneOf (8, 3, 1))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be ("1") or not contain oneOf ('2', '3', '1'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", and " + decorateToStringValue("1") + " contained one of ('2', '3', '1')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be ("o") or not contain oneOf ('i'))
        all (hiLists) should (not be ("i") or not contain oneOf ('i'))
        all (hiLists) should (not be ("o") or not contain oneOf ('o'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") or not contain oneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", and " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be ("o") or not contain oneOf ('i'))) (decided by invertedStringEquality)
        (all (hiLists) should (not be ("i") or not contain oneOf ('i'))) (decided by invertedStringEquality)
        (all (hiLists) should (not be ("o") or not contain oneOf ('o'))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("i") or not contain oneOf ('o'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", and " + decorateToStringValue("i") + " contained one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
