/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainOnlyLogicalAndSpec extends Spec with Matchers {
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case e: java.util.Map.Entry[_, _] => 
        (e.getKey, e.getValue) match {
          case (k: String, v: String) => Entry(k.toUpperCase, v.toUpperCase)
          case _ => value
        }
      case _ => value
    }
  
  val upperCaseListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainOnlyLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: String = "upie"
    val toList: String = "yobh"
    
    object `when used with (contain only (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain only ('e', 'i', 'p', 'u') and contain only ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('h', 'b', 'o', 'y') and contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain only ('e', 'i', 'p', 'u') and contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain only ('E', 'I', 'P', 'U') and contain only ('E', 'I', 'U', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('E', 'I', 'P', 'A') and contain only ('E', 'I', 'U', 'P'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain only ('E', 'I', 'P', 'U') and (contain only ('E', 'I', 'A', 'P')))
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'A', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain only ('E', 'I', 'P', 'U') and contain only ('E', 'I', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain only ('E', 'I', 'P', 'A') and contain only ('E', 'I', 'U', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain only ('E', 'I', 'P', 'U') and contain only ('E', 'I', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'U'") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'A', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') and contain only ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (equal (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain only ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) and contain only ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain only ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain only ('E', 'I', 'P', 'A')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain only ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain only ('E', 'I', 'P', 'A'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'E', 'I', 'P', 'A'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain only ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain only ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    object `when used with (legacyEqual (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain only ('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) and contain only ('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain only ('E', 'I', 'P', 'U'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain only ('h', 'b', 'o', 'y')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain only ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain only ('h', 'b', 'o', 'y'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain only ('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain only ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
    }

    object `when used with (contain only (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain only ('i', 'e', 'u', 'p') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('e', 'i', 'p', 'u') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain only ('h', 'b', 'o', 'y') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain only ('I', 'E', 'U', 'P') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain only ('I', 'E', 'A', 'P') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain only ('H', 'B', 'O', 'Y') and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'H', 'B', 'O', 'Y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain only ('I', 'E', 'U', 'P') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain only ('I', 'E', 'A', 'P') and legacyEqual (fumList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain only ('I', 'E', 'U', 'P') and legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain only ("fee", "fie", "foe", "fuu") and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain only ('e', 'i', 'p', 'u') and not contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain only ('h', 'b', 'o', 'y') and not contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain only ('I', 'E', 'A', 'P') and not contain only ('I', 'E', 'P', 'A'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain only ('I', 'E', 'U', 'P') and not contain only ('I', 'E', 'P', 'A'))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain only ('I', 'E', 'A', 'P') and (not contain only ('I', 'E', 'P', 'U')))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'P', 'U'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain only ('I', 'E', 'A', 'P') and not contain only ('I', 'E', 'P', 'A'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain only ('I', 'E', 'A', 'P') and not contain only ('I', 'E', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'A', 'P'") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'P', 'U'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain only ('I', 'E', 'U', 'P') and not contain only ('I', 'E', 'P', 'A'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        (fumList should (contain only ('E', 'I', 'P', 'U') and contain only ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not equal (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) and not contain only ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain only ('I', 'E', 'A', 'P'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain only ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain only ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain only ('I', 'E', 'U', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain only ('I', 'E', 'A', 'P'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain only ('E', 'I', 'P', 'D') and not contain only ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain only ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain only ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) and not contain only ('I', 'E', 'A', 'P'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain only ('I', 'E', 'A', 'P'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain only ('I', 'E', 'U', 'P')))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain only ('I', 'E', 'U', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumList), "'I', 'E', 'U', 'P'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain only ('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain only ('E', 'I', 'P', 'D') and not contain only ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  object `col of Lists` {
    
    val list1s: Vector[String] = Vector("321", "321", "321")
    val lists: Vector[String] = Vector("321", "321", "432")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("321", "321", "")
    val hiLists: Vector[String] = Vector("il", "il", "il")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain only xx and contain only xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only ('3', '2', '1') and contain only ('1', '3', '2'))
        atLeast (2, lists) should (contain only ('3', '1', '2') and contain only ('2', '3', '1'))
        atMost (2, lists) should (contain only ('3', '1', '2') and contain only ('2', '3', '1'))
        no (lists) should (contain only ('3', '6', '9') and contain only ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only ('1', '2', '3') and contain only ('1', '3', '2'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("432") + " did not contain only " + "('1', '2', '3')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain only ('1', '2', '3') and contain only ('1', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("321") + " contained only " + "('1', '2', '3')" + ", but " + decorateToStringValue("321") + " did not contain only " + "('1', '3', '4')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain only ('i', 'l') and contain only ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('i', 'l')" + ", but " + decorateToStringValue("il") + " did not contain only " + "('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain only ('L', 'I') and contain only ('I', 'L'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain only ('O', 'L') and contain only ('I', 'L'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain only ('L', 'I') and contain only ('O', 'L'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')" + ", but " + decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain only ('L', 'I') and contain only ('I', 'L'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain only ('O', 'L') and contain only ('I', 'L'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain only ('L', 'I') and contain only ('O', 'L'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')" + ", but " + decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("321") and contain only ('1', '3', '2'))
        atLeast (2, lists) should (be ("321") and contain only ('1', '3', '2'))
        atMost (2, lists) should (be ("321") and contain only ('2', '3', '1'))
        no (lists) should (be (List('3', '6', '9')) and contain only ('3', '4', '5'))
        no (nils) should (be (List('1', '6', '8')) and contain only ('1', '3', '4'))
        no (listsString) should (be (List('2', '6', '8')) and contain only ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be ("321") and contain only ('1', '3', '2'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("[432]") + " was not equal to " + decorateToStringValue("[321]"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be ("321") and contain only ('2', '3', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("321") + " was equal to " + decorateToStringValue("321") + ", but " + decorateToStringValue("321") + " did not contain only " + "('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be ("e") and contain only ('l', 'i'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[e]"), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be ("il") and contain only ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", but " + decorateToStringValue("il") + " did not contain only " + "('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsString) should (be ("321") and contain only ('1', '3', '2'))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[321]"), thisLineNumber - 2, listsString), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be ("321") and contain only ('2', '3', '8'))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue("321") + " was equal to " + decorateToStringValue("321") + ", but " + decorateToStringValue("321") + " did not contain only " + "('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("il") and contain only ('L', 'I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (List('I', 'L')) and contain only ('L', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('I', 'L')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be ("il") and contain only ('O', 'L'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", but " + decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("il") and contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (List('I', 'L')) and contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('I', 'L')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be ("il") and contain only ('O', 'L'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", but " + decorateToStringValue("il") + " did not contain only " + "('O', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain only ('3', '2', '8') and not contain only ('8', '3', '4'))
        atLeast (2, lists) should (not contain only ('3', '8', '5') and not contain only ('8', '3', '4'))
        atMost (2, lists) should (not contain only (2, 4, 3) and contain only ('4', '3', '2'))
        no (list1s) should (not contain only ('1', '2', '3') and not contain only ('1', '3', '2'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain only ('2', '3', '4') and not contain only ('8', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("432") + " contained only " + "('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain only ('3', '6', '8') and not contain only ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("432") + " did not contain only " + "('3', '6', '8')" + ", but " + decorateToStringValue("432") + " contained only " + "('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain only ('l', 'i') and not contain only ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('l', 'i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain only ('o', 'e', 'd') and not contain only ('l', 'i'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('o', 'e', 'd')" + ", but " + decorateToStringValue("il") + " contained only " + "('l', 'i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain only ('I') and not contain only ('O'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain only ('L', 'I') and not contain only ('O'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain only ('I') and not contain only ('L', 'I'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('I')" + ", but " + decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain only ('I') and not contain only ('O'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain only ('L', 'I') and not contain only ('O'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain only ('I') and not contain only ('L', 'I'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("il") + " did not contain only " + "('I')" + ", but " + decorateToStringValue("il") + " contained only " + "('L', 'I')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") and not contain only ('8', '3', '4'))
        atLeast (2, lists) should (not be ("3") and not contain only ('8', '3', '4'))
        atMost (2, lists) should (not be ("432") and not contain only ('3', '4', '2'))
        no (list1s) should (not be ("321") and not contain only ('1', '2', '3'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be ("432") and not contain only ('8', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("432") + " was equal to " + decorateToStringValue("432"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be ("3") and not contain only ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("[432]") + " was not equal to " + decorateToStringValue("[3]") + ", but " + decorateToStringValue("432") + " contained only " + "('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be ("il") and not contain only ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain only ('l', 'i'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("[il]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("il") + " contained only " + "('l', 'i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be ("o") and not contain only ('O', 'L'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("il") and not contain only ('L', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain only ('I', 'L'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[il]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("il") + " contained only " + "('I', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be ("o") and not contain only ('O', 'L'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("il") and not contain only ('L', 'I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("o") and not contain only ('I', 'L'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[il]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("il") + " contained only " + "('I', 'L')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
